@@ -6,13 +6,24 @@ from docutils.parsers.rst import Directive
 from sphinx import addnodes
 from sphinx.errors import SphinxError
 from sphinx.util.nodes import split_explicit_title, process_index_entry, set_role_source_info
-from htc_helpers import make_headerlink_node
+from htc_helpers import make_headerlink_node, warn
+
+SUBMIT_CMD_DEFS = []
 
 def dump(obj):
     for attr in dir(obj):
         print("obj.%s = %r" % (attr, getattr(obj, attr)))
 
 def subcom_def_role(name, rawtext, text, lineno, inliner, options={}, content=[]):
+    global SUBMIT_CMD_DEFS
+    if text in SUBMIT_CMD_DEFS:
+        docname = inliner.document.settings.env.docname
+        warn(f"{docname} @ {lineno} | '{text}' submit command already defined!")
+        textnode = nodes.Text(text, " ")
+        return [textnode], []
+    else:
+        SUBMIT_CMD_DEFS.append(text)
+
     # Create a new linkable target using the subcom name
     targetid = text
     targetnode = nodes.target('', text, ids=[targetid], classes=["subcom-def"])
