@@ -558,6 +558,7 @@ our %submit_info = (
 	'nmi-build:x86_64_Fedora32'		=> 'x86_64_Fedora',
 	'nmi-build:x86_64_Fedora38'		=> 'x86_64_Fedora',
 	'nmi-build:x86_64_Fedora39'		=> 'x86_64_Fedora',
+	'nmi-build:x86_64_Fedora40'		=> 'x86_64_Fedora',
 	'nmi-build:x86_64_AmazonLinux2023' => 'x86_64_Fedora',
 	'nmi-build:x86_64_openSUSE15'   => 'x86_64_Fedora',
 	
@@ -671,6 +672,7 @@ our %submit_info = (
 	'nmi-build:x86_64_Ubuntu18'	=> 'x86_64_Ubuntu18',
 	'nmi-build:x86_64_Ubuntu20'	=> 'x86_64_Ubuntu18',
 	'nmi-build:x86_64_Ubuntu22'	=> 'x86_64_Ubuntu18',
+	'nmi-build:x86_64_Ubuntu24'	=> 'x86_64_Ubuntu18',
 	'nmi-build:ppc64le_Ubuntu20'	=> 'x86_64_Ubuntu18',
 
 	# Add the SWAMP's (temporary) platform name
