@@ -330,14 +330,9 @@ InsertAttr( const string &name, long long value)
 	// Optimized insert of long long values that overwrite the destination value if the destination is a literal.
 	classad::ExprTree* & expr = attrList[name];
 	if (expr) {
-		if (expr->GetKind() == LITERAL_NODE) {
-			((Literal*)expr)->SetLong(value);
-			return true;
-		} else {
 			delete expr;
-		}
 	}
-	expr = Literal::MakeLong(value);
+	expr = Literal::MakeInteger(value);
 	return expr != NULL;
 }
 
@@ -396,12 +391,7 @@ InsertAttr( const string &name, double value)
 	// Optimized insert of Real values that overwrite the destination value if the destination is a literal.
 	classad::ExprTree* & expr = attrList[name];
 	if (expr) {
-		if (expr->GetKind() == LITERAL_NODE) {
-			((Literal*)expr)->SetReal(value);
-			return true;
-		} else {
 			delete expr;
-		}
 	}
 	expr = Literal::MakeReal(value);
 	return expr != NULL;
@@ -429,12 +419,7 @@ InsertAttr( const string &name, bool value )
 	// Optimized insert of bool values that overwrite the destination value if the destination is a literal.
 	classad::ExprTree* & expr = attrList[name];
 	if (expr) {
-		if (expr->GetKind() == LITERAL_NODE) {
-			((Literal*)expr)->SetBool(value);
-			return true;
-		} else {
 			delete expr;
-		}
 	}
 	expr = Literal::MakeBool(value);
 	return expr != NULL;
@@ -468,12 +453,7 @@ InsertAttr( const string &name, const char * str, size_t len)
 	// Optimized insert of long long values that overwrite the destination value if the destination is a literal.
 	classad::ExprTree* & expr = attrList[name];
 	if (expr) {
-		if (expr->GetKind() == LITERAL_NODE) {
-			((Literal*)expr)->SetString(str, len);
-			return true;
-		} else {
 			delete expr;
-		}
 	}
 	expr = Literal::MakeString( str, len );
 	return expr != NULL;
@@ -512,7 +492,7 @@ bool ClassAd::Insert(const std::string& serialized_nvp);
 
 // Parse and insert an attribute value via cache if the cache is enabled
 //
-bool ClassAd::InsertViaCache( std::string& name, const std::string & rhs, bool lazy /*=false*/)
+bool ClassAd::InsertViaCache(const std::string& name, const std::string & rhs, bool lazy /*=false*/)
 {
 	if (name.empty()) return false;
 
@@ -941,28 +921,23 @@ Modify( ClassAd& mod )
 		// Step 3:  Process Deletes attribute
 	if( ( expr = mod.Lookup( ATTR_DELETES ) ) != NULL ) {
 		const ExprList 		*list;
-		ExprListIterator	itor;
 		const char			*attrName;
 
 			// make a first pass to check that it is a list of strings ...
 		if( !expr->Evaluate( val ) || !val.IsListValue( list ) ) {
 			return;
 		}
-		itor.Initialize( list );
-		while( ( expr = itor.CurrentExpr( ) ) ) {
+		for (const auto *expr: *list) {
 			if( !expr->Evaluate( val ) || !val.IsStringValue( attrName ) ) {
 				return;
 			}
-			itor.NextExpr( );
 		}
 
 			// now go through and delete all the named attributes ...
-		itor.Initialize( list );
-		while( ( expr = itor.CurrentExpr( ) ) ) {
+		for (const auto *expr: *list) {
 			if( expr->Evaluate( val ) && val.IsStringValue( attrName ) ) {
 				ctx->Delete( attrName );
 			}
-			itor.NextExpr( );
 		}
 	}
 
@@ -1169,7 +1144,6 @@ EvaluateExpr( const ExprTree *tree , Value &val, Value::ValueType mask ) const
 	return res;
 }
 
-
 bool ClassAd::
 EvaluateExpr( const ExprTree *tree , Value &val , ExprTree *&sig ) const
 {
@@ -1267,37 +1241,6 @@ EvaluateAttrBoolEquiv( const string &attr, bool &b ) const
 	return( EvaluateAttr( attr, val, Value::ValueType::NUMBER_VALUES ) && val.IsBooleanValueEquiv( b ) );
 }
 
-#if 0
-// disabled (see header)
-bool ClassAd::
-EvaluateAttrClassAd( const string &attr, ClassAd *&classad ) const
-{
-	Value val;
-		// TODO: filter out shared_ptr<ClassAd> values that would
-		// go out of scope here (if such a thing is ever added),
-		// or return a shared_ptr and make a copy here of the
-		// ClassAd if it is not already managed by a shared_ptr.
-	return( EvaluateAttr( attr, val ) && val.IsClassAdValue( classad ) );
-}
-#endif
-
-#if 0
-// disabled (see header)
-bool ClassAd::
-EvaluateAttrList( const string &attr, ExprList *&l ) const
-{
-    Value val;
-		// This version of EvaluateAttrList() can only succeed
-		// if the result is LIST_VALUE, not SLIST_VALUE, because
-		// the shared_ptr<ExprList> goes out of scope before
-		// returning to the caller.  Either do as below and filter
-		// out SLIST_VALUE, or return a shared_ptr and create a
-		// copy of the list here if it is not already managed
-		// by a shared_ptr.
-	return( EvaluateAttr( attr, val ) && val.GetType() == LIST_VALUE && val.IsListValue( l ) );
-}
-#endif
-
 bool ClassAd::
 GetExternalReferences( const ExprTree *tree, References &refs, bool fullNames ) const
 {
@@ -1317,8 +1260,15 @@ bool ClassAd::
 _GetExternalReferences( const ExprTree *expr, const ClassAd *ad, 
 	EvalState &state, References& refs, bool fullNames )
 {
-    switch( expr->GetKind( ) ) {
-        case LITERAL_NODE:
+    switch( expr->GetKind()) {
+		case ERROR_LITERAL:
+		case UNDEFINED_LITERAL:
+		case BOOLEAN_LITERAL:
+		case INTEGER_LITERAL:
+		case REAL_LITERAL:
+		case RELTIME_LITERAL:
+		case ABSTIME_LITERAL:
+		case STRING_LITERAL:
                 // no external references here
             return( true );
 
@@ -1519,7 +1469,14 @@ _GetExternalReferences( const ExprTree *expr, const ClassAd *ad,
 	EvalState &state, PortReferences& refs ) const
 {
     switch( expr->GetKind( ) ) {
-        case LITERAL_NODE:
+		case ERROR_LITERAL:
+		case UNDEFINED_LITERAL:
+		case BOOLEAN_LITERAL:
+		case INTEGER_LITERAL:
+		case REAL_LITERAL:
+		case RELTIME_LITERAL:
+		case ABSTIME_LITERAL:
+		case STRING_LITERAL:
                 // no external references here
             return( true );
 
@@ -1677,10 +1634,16 @@ _GetInternalReferences( const ExprTree *expr, const ClassAd *ad,
 
     switch( expr->GetKind() ){
         //nothing to be found here!
-        case LITERAL_NODE:{
+		case ERROR_LITERAL:
+		case UNDEFINED_LITERAL:
+		case BOOLEAN_LITERAL:
+		case INTEGER_LITERAL:
+		case REAL_LITERAL:
+		case RELTIME_LITERAL:
+		case ABSTIME_LITERAL:
+		case STRING_LITERAL:
             return true;
         break;
-                          }
 
         case ATTRREF_NODE:{
             const ClassAd   *start;
@@ -1888,7 +1851,6 @@ _GetInternalReferences( const ExprTree *expr, const ClassAd *ad,
             return false;
 
     }
-
 }
 
 #if defined( EXPERIMENTAL )
@@ -2004,15 +1966,15 @@ _MakeRectangles( const ExprTree *tree, const string &allowed, Rectangles &r,
 		return( false );
 	}
 
-	if( t1->GetKind( )==ExprTree::ATTRREF_NODE && 
-			t2->GetKind( )==ExprTree::LITERAL_NODE ) {
+	if (t1->GetKind( )==ExprTree::ATTRREF_NODE && 
+			dynamic_cast<Literal *>(t2) != nullptr) {
 			// ref <op> lit
 		attr = t1;
 		lit  = t2;
 			// if not the attribute we're interested in, ignore
 		if( !_CheckRef( attr, allowed ) ) return( true );
 	} else if(t2->GetKind()==ExprTree::ATTRREF_NODE && 
-			t1->GetKind()==ExprTree::LITERAL_NODE){
+			dynamic_cast<Literal *>(t1) != nullptr) {
 			// lit <op> ref
 		attr = t2;
 		lit  = t1;
