@@ -156,7 +156,7 @@ Evaluate (EvalState &state, Value &val) const
 	}
 #endif
 
-	if(state.debug && GetKind() != ExprTree::LITERAL_NODE &&
+	if(state.debug && (dynamic_cast<const Literal *>(this) == nullptr) &&
 			GetKind() != ExprTree::OP_NODE)
 	{
 		debug_format_value(val, diff);
@@ -184,7 +184,7 @@ Evaluate( EvalState &state, Value &val, ExprTree *&sig ) const
 	}
 #endif
 
-	if(state.debug && GetKind() != ExprTree::LITERAL_NODE &&
+	if(state.debug && (dynamic_cast<const Literal *>(this) == nullptr) &&
 			GetKind() != ExprTree::OP_NODE)
 	{
 		debug_format_value(val, diff);
