@@ -26,6 +26,7 @@
 #include "daemon.h"
 #include "condor_daemon_core.h"
 #include "dc_collector.h"
+#include "subsystem_info.h"
 
 #include <algorithm>
 
@@ -250,12 +251,10 @@ DCCollector::parseTCPInfo( void )
 		use_tcp = false;
 		char *tmp = param( "TCP_UPDATE_COLLECTORS" );
 		if( tmp ) {
-			StringList tcp_collectors;
-
-			tcp_collectors.initializeFromString( tmp );
+			std::vector<std::string> tcp_collectors = split(tmp);
 			free( tmp );
 			if( ! _name.empty() &&
-				tcp_collectors.contains_anycase_withwildcard(_name.c_str()) )
+				contains_anycase_withwildcard(tcp_collectors, _name) )
 			{	
 				use_tcp = true;
 				break;
@@ -273,6 +272,11 @@ DCCollector::parseTCPInfo( void )
 	}
 }
 
+// do a >= version check 
+bool DCCollector::checkCachedVersion(int major, int minor, int subminor, bool default_value) {
+	if (_version.empty()) return default_value;
+	return CondorVersionInfo(_version.c_str()).built_since_version(major, minor, subminor);
+}
 
 bool
 DCCollector::sendUpdate( int cmd, ClassAd* ad1, DCCollectorAdSequences& adSeq, ClassAd* ad2, bool nonblocking, StartCommandCallbackType callback_fn, void *miscdata)
@@ -289,6 +293,23 @@ DCCollector::sendUpdate( int cmd, ClassAd* ad1, DCCollectorAdSequences& adSeq, C
 		nonblocking = false;
 	}
 
+	// if we have not yet stashed the collector version from the sock into the daemon object do that now.
+	// the update sock version is set by security negotiatiation, so we won't know it for the first update.
+	// By the time we *do* know it we are committed to the command int <sigh>. Because of this it is the
+	// callers responsibility to make sure that the first command over the update sock is safe for
+	// all versions of the collector.
+	// The adtype INVALIDATE commands are safe, the adtype UPDATE commands are only safe if adtype
+	// is one of the adtypes that has been around forever *and* the ad itself is one that the collector
+	// is willing to store.  Thus it is *not* safe to a STARTD daemon ad to be the first update, because
+	// a pre 23.2 collector will either store it in the slot ad table, or it will close the connection if
+	// it cannot be hashed as a slot ad.
+	if (_version.empty() && update_rsock) {
+		auto * verinfo = update_rsock->get_peer_version();
+		if (verinfo) { _version = verinfo->get_version_stdstring(); }
+		dprintf(D_ZKM, "DCCollector::sendUpdate collector %s version was unknown, is now %s\n",
+			_name.c_str(), _version.c_str());
+	}
+
 	// Add start time & seq # to the ads before we publish 'em
 	if ( ad1 ) {
 		ad1->Assign(ATTR_DAEMON_START_TIME, startTime);
@@ -300,12 +321,33 @@ DCCollector::sendUpdate( int cmd, ClassAd* ad1, DCCollectorAdSequences& adSeq, C
 	}
 
 	if ( ad1 ) {
-		DCCollectorAdSeq* seqgen = adSeq.getAdSeq(*ad1);
-		if (seqgen) {
-			long long seq = seqgen->getSequence();
-			ad1->Assign(ATTR_UPDATE_SEQUENCE_NUMBER, seq);
-			if (ad2) { ad2->Assign(ATTR_UPDATE_SEQUENCE_NUMBER, seq); }
+		auto & seqgen = adSeq.getAdSeq(*ad1);
+		if (cmd == UPDATE_STARTD_AD && seqgen.getAdType() == STARTDAEMON_AD
+			&& do_version_check_before_startd_daemon_ad_update) {
+			// We can't send STARTD daemon ads to collectors that are older than 23.2
+			// so when we don't know the version, or the collector is older, just fail now
+			const char * err_reason = nullptr;
+			if (_version.empty()) {
+				err_reason = "version is not known";
+			} else if ( ! CondorVersionInfo(_version.c_str()).built_since_version(23,2,0)) {
+				err_reason = "version is older than 23.2";
+			}
+			if (err_reason) {
+				std::string err_msg, adname;
+				ad1->LookupString(ATTR_NAME, adname);
+				formatstr(err_msg, "Collector %s %s - will not send STARD daemon ad %s",
+					_name.c_str(), err_reason, adname.c_str());
+				newError(CA_INVALID_REQUEST, err_msg.c_str());
+				if (callback_fn) {
+					(*callback_fn)(false, nullptr, nullptr, "", false, miscdata);
+				}
+				dprintf(D_ZKM, "DCCollector::sendUpdate will not send STARTD daemon ad because %s\n", err_reason);
+				return false;
+			}
 		}
+		long long seq = seqgen.getSequence();
+		ad1->Assign(ATTR_UPDATE_SEQUENCE_NUMBER, seq);
+		if (ad2) { ad2->Assign(ATTR_UPDATE_SEQUENCE_NUMBER, seq); }
 	}
 
 		// Prior to 7.2.0, the negotiator depended on the startd
@@ -364,7 +406,7 @@ DCCollector::sendUpdate( int cmd, ClassAd* ad1, DCCollectorAdSequences& adSeq, C
 				return false;
 			}
 			if( strcmp( myOwnSinful, _addr.c_str() ) == 0 ) {
-				EXCEPT( "Collector attempted to send itself an update.\n" );
+				EXCEPT( "Collector attempted to send itself an update." );
 			}
 		}
 	}
@@ -385,8 +427,11 @@ DCCollector::finishUpdate( DCCollector *self, Sock* sock, ClassAd* ad1, ClassAd*
 		// to share submitter secrets.
 	auto *verinfo = sock->get_peer_version();
 	bool send_submitter_secrets = false;
-	if (verinfo && verinfo->built_since_version(8, 9, 3)) {
-		send_submitter_secrets = true;
+	if (verinfo) {
+		if (self && self->_version.empty()) { self->_version = verinfo->get_version_stdstring(); };
+		if (verinfo->built_since_version(8, 9, 3)) {
+			send_submitter_secrets = true;
+		}
 	}
 
 		// If we are advertising to an admin-configured pool, then
@@ -737,8 +782,13 @@ DCCollector::initiateTCPUpdate( int cmd, ClassAd* ad1, ClassAd* ad2, bool nonblo
 void
 DCCollector::displayResults( void )
 {
-	dprintf( D_FULLDEBUG, "Will use %s to update collector %s\n", 
+	// Write to log file the collector receiving updates.
+	// Note: the gangliad does not do this, as it is misleading and
+	// worries admin (gangliad supposedly only reading collectors, not updating)
+	if (strcmp(get_mySubSystem()->getName(),"GANGLIAD")) {
+		dprintf( D_FULLDEBUG, "Will use %s to update collector %s\n", 
 			 use_tcp ? "TCP" : "UDP", updateDestination() );
+	}
 }
 
 
@@ -782,20 +832,24 @@ DCCollector::initDestinationStrings( void )
 //
 
 // Get a sequence number class for this classad, creating it if needed.
-DCCollectorAdSeq* DCCollectorAdSequences::getAdSeq(const ClassAd & ad)
+DCCollectorAdSeq& DCCollectorAdSequences::getAdSeq(const ClassAd & ad)
 {
+	AdTypes mytype = NO_AD;
 	std::string name, attr;
 	ad.LookupString( ATTR_NAME, name );
 	ad.LookupString( ATTR_MY_TYPE, attr );
 	name += "\n"; name += attr;
+	if ( ! attr.empty()) { mytype = AdTypeStringToAdType(attr.c_str()); }
 	ad.LookupString( ATTR_MACHINE, attr );
 	name += "\n"; name += attr;
 
 	DCCollectorAdSeqMap::iterator it = seqs.find(name);
 	if (it != seqs.end()) {
-		return &(it->second);
+		return it->second;
 	}
-	return &(seqs[name]);
+	auto & seq = seqs[name];
+	seq.setAdType(mytype);
+	return seq;
 }
 
 DCCollector::~DCCollector( void )
