@@ -196,6 +196,7 @@
 #define ATTR_ENTERED_CURRENT_STATUS  "EnteredCurrentStatus"
 #define ATTR_ERROR_STRING  "ErrorString"
 #define ATTR_ERROR_CODE  "ErrorCode"
+#define ATTR_WARNING "Warning"
 #define ATTR_EXCEPTION_HIERARCHY  "ExceptionHierarchy"
 #define ATTR_EXCEPTION_NAME  "ExceptionName"
 #define ATTR_EXCEPTION_TYPE  "ExceptionType"
@@ -302,6 +303,8 @@
 #define ATTR_UNHIBERNATE  "Unhibernate"
 #define ATTR_HOLD_KILL_SIG  "HoldKillSig"
 #define ATTR_HOOK_KEYWORD  "HookKeyword"
+#define ATTR_HISTORY_RECORD_SOURCE  "HistoryRecordSource"
+#define ATTR_HISTORY_AD_TYPE_FILTER  "HistoryAdTypeFilter"
 #define ATTR_IDLE_JOBS  "IdleJobs"
 #define ATTR_IMAGE_SIZE  "ImageSize"
 #define ATTR_IO_WAIT  "IOWait"
@@ -331,6 +334,9 @@
 #define ATTR_JOB_ALLOWED_JOB_DURATION  "AllowedJobDuration"
 #define ATTR_JOB_ALLOWED_EXECUTE_DURATION "AllowedExecuteDuration"
 #define ATTR_JOB_EPOCH_WRITE_DATE "EpochWriteDate"
+#define ATTR_JOB_LAST_SHADOW_EXCEPTION    "LastShadowException"
+#define ATTR_JOB_COOL_DOWN_EXPIRATION "JobCoolDownExpiration"
+#define ATTR_NUM_JOB_COOL_DOWNS "NumJobCoolDowns"
 #define ATTR_JOB_MATERIALIZE_DIGEST_FILE  "JobMaterializeDigestFile"
 #define ATTR_JOB_MATERIALIZE_ITEMS_FILE   "JobMaterializeItemsFile"
 #define ATTR_JOB_MATERIALIZE_LIMIT        "JobMaterializeLimit"
@@ -638,6 +644,10 @@
 #define ATTR_REQUEUE_REASON  "RequeueReason"
 #define ATTR_REQUIREMENTS  "Requirements"
 #define ATTR_CUDA_VERSION "CUDAVersion"
+#define ATTR_QUERY_Q_IDS                "IDs"
+#define ATTR_QUERY_Q_INCLUDE_CLUSTER_AD "IncludeClusterAd"
+#define ATTR_QUERY_Q_INCLUDE_JOBSET_ADS "IncludeJobsetAds"
+#define ATTR_QUERY_Q_NO_PROC_ADS        "NoProcAds"
 #define ATTR_RESOURCE_REQUEST_CONSTRAINT "_condor_RESOURCE_CONSTRAINT"  // used in resource request ad
 #define ATTR_RESOURCE_REQUEST_COUNT "_condor_RESOURCE_COUNT"  // used in resource request ad
 #define ATTR_RESOURCE_REQUEST_CLUSTER "_condor_RESOURCE_CLUSTER"
@@ -806,6 +816,9 @@
 #define ATTR_USERREC_OPT_ME "_userrec_opt_me"
 #define ATTR_USERREC_OPT_CREATE_DEPRECATED "create"
 #define ATTR_VACATE  "Vacate"
+#define ATTR_VACATE_REASON  "VacateReason"
+#define ATTR_VACATE_REASON_CODE  "VacateReasonCode"
+#define ATTR_VACATE_REASON_SUBCODE  "VacateReasonSubCode"
 #define ATTR_VACATE_TYPE  "VacateType"
 #define ATTR_VIRTUAL_MEMORY  "VirtualMemory"
 #define ATTR_VISA_TIMESTAMP  "VisaTimestamp"
@@ -1060,6 +1073,7 @@ extern const char ATTR_SEC_AUTHENTICATED_USER [];
 #define ATTR_REQUEST_DISK  "RequestDisk"
 #define ATTR_REQUEST_GPUS  "RequestGPUs"
 #define ATTR_REQUIRE_GPUS  "RequireGPUs"
+#define ATTR_REQUIRE_PREFIX "Require"
 #define ATTR_REQUEST_VIRTUAL_MEMORY  "RequestVirtualMemory"
 // GPUs property constraints
 #define ATTR_GPUS_MIN_MEMORY "GPUsMinMemory"
