@@ -30,6 +30,9 @@
 #include "condor_netdb.h"
 #include "selector.h"
 #include "condor_sockfunc.h"
+#include "condor_sinful.h"
+#include "ipv6_hostname.h"
+#include "condor_daemon_core.h"
 
 _condorMsgID SafeSock::_outMsgID = {0, 0, 0, 0};
 unsigned long SafeSock::_noMsgs = 0;
@@ -975,3 +978,25 @@ SafeSock::recvQueueDepth(int port) {
 #endif
 	return depth;
 }
+
+int
+SafeSock::do_reverse_connect(char const *,bool,CondorError *)
+{
+	dprintf(D_ALWAYS,
+			"CCBClient: WARNING: UDP not supported by CCB."
+			"  Will therefore try to send packet directly to %s.\n",
+			peer_description());
+
+	return CEDAR_ENOCCB;
+}
+
+int
+SafeSock::do_shared_port_local_connect( char const *, bool, char const * )
+{
+	dprintf(D_ALWAYS,
+			"SharedPortClient: WARNING: UDP not supported."
+			"  Failing to connect to %s.\n",
+			peer_description());
+
+	return 0;
+}
