@@ -25,7 +25,6 @@
 #include "condor_accountant.h"
 #include "condor_io.h"
 #include "HashTable.h"
-#include "string_list.h"
 #include "dc_collector.h"
 #include "condor_ver_info.h"
 #include "matchmaker_negotiate.h"
@@ -351,8 +350,7 @@ class Matchmaker : public Service
 
 		bool m_staticRanks;
 
-		StringList NegotiatorMatchExprNames;
-		StringList NegotiatorMatchExprValues;
+		std::map<std::string, std::string> NegotiatorMatchExprs;
 
 		std::map<std::string, int> ScheddsTimeInCycle;
 
