@@ -45,7 +45,6 @@
 #include "error_utils.h"
 #include "print_wrapped_text.h"
 #include "condor_distribution.h"
-#include "string_list.h"
 #include "condor_version.h"
 #include "subsystem_info.h"
 #include "condor_open.h"
@@ -55,7 +54,6 @@
 #include "ipv6_hostname.h"
 #include <map>
 #include <vector>
-//#include "../classad_analysis/analysis.h"
 //#include "pool_allocator.h"
 #include "expr_analyze.h"
 #include "classad/classadCache.h" // for CachedExprEnvelope stats
@@ -745,6 +743,8 @@ bool doJobRunAnalysis (
 	bool	val;
 	int		universe = CONDOR_UNIVERSE_MIN;
 	int		jobState;
+	time_t  cool_down = 0;
+	time_t  server_time = time(nullptr);
 	std::string owner;
 	std::string user;
 	std::string slotname;
@@ -763,6 +763,8 @@ bool doJobRunAnalysis (
 	request->LookupInteger(ATTR_LAST_REJ_MATCH_TIME, last_rej_match_time);
 
 	request->LookupInteger(ATTR_JOB_STATUS, jobState);
+	request->LookupInteger(ATTR_JOB_COOL_DOWN_EXPIRATION, cool_down);
+	request->LookupInteger(ATTR_SERVER_TIME, server_time);
 	if (jobState == RUNNING || jobState == TRANSFERRING_OUTPUT || jobState == SUSPENDED) {
 		job_status = "Job is running.";
 	}
@@ -781,6 +783,9 @@ bool doJobRunAnalysis (
 	if (jobState == COMPLETED) {
 		job_status = "Job is completed.";
 	}
+	if (jobState == IDLE && cool_down > server_time) {
+		job_status = "Job is in cool down.";
+	}
 
 	// if we already figured out the job status, and we haven't been asked to analyze requirements anyway
 	// we are done.
@@ -1409,6 +1414,7 @@ const char * doSlotRunAnalysisToBuffer(ClassAd *slot, JobClusterMap & clusters,
 		if ( ! (analyze_detail_level & detail_inline_std_slot_exprs)) {
 			inline_attrs.clear();
 			inline_attrs.insert(ATTR_START);
+			inline_attrs.insert(ATTR_WITHIN_RESOURCE_LIMITS);
 		}
 
 		std::string subexpr_detail;
