@@ -39,6 +39,8 @@
 #include "ipv6_hostname.h"
 #include <math.h>
 #include "job_ad_instance_recording.h"
+#include "../condor_sysapi/sysapi.h"
+#include <algorithm>
 
 // these are declared static in baseshadow.h; allocate space here
 BaseShadow* BaseShadow::myshadow_ptr = NULL;
@@ -94,6 +96,8 @@ void
 BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer_queue_contact_info )
 {
 	int pending = FALSE;
+	const char * owner = nullptr; // bare username, even when USERREC_NAME_IS_FULLY_QUALIFIED
+	std::string ownerbuf;
 
 	if( ! job_ad ) {
 		EXCEPT("baseInit() called with NULL job_ad!");
@@ -107,9 +111,15 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 
 	m_xfer_queue_contact_info = xfer_queue_contact_info ? xfer_queue_contact_info : "";
 
-	if ( !jobAd->LookupString(ATTR_OWNER, owner)) {
+	if (USERREC_NAME_IS_FULLY_QUALIFIED) {
+		if ( !jobAd->LookupString(ATTR_USER, user_owner)) {
+			EXCEPT("Job ad doesn't contain an %s attribute.", ATTR_USER);
+		}
+	} else
+	if ( !jobAd->LookupString(ATTR_OWNER, user_owner)) {
 		EXCEPT("Job ad doesn't contain an %s attribute.", ATTR_OWNER);
 	}
+	owner = name_of_user(user_owner.c_str(), ownerbuf);
 
 	if( !jobAd->LookupInteger(ATTR_CLUSTER_ID, cluster)) {
 		EXCEPT("Job ad doesn't contain a %s attribute.", ATTR_CLUSTER_ID);
@@ -170,8 +180,8 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 	// Calling init_user_ids() while in user priv causes badness.
 	// Make sure we're in another priv state.
 	set_condor_priv();
-	if ( !init_user_ids(owner.c_str(), domain.c_str())) {
-		dprintf(D_ALWAYS, "init_user_ids() failed as user %s\n",owner.c_str() );
+	if ( !init_user_ids(owner, domain.c_str())) {
+		dprintf(D_ALWAYS, "init_user_ids() failed as user %s\n", owner);
 		// uids.C will EXCEPT when we set_user_priv() now
 		// so there's not much we can do at this point
 		
@@ -182,7 +192,7 @@ BaseShadow::baseInit( ClassAd *job_ad, const char* schedd_addr, const char *xfer
 			
 			owner="nobody";
 			domain="";
-			if (!init_user_ids(owner.c_str(), domain.c_str()))
+			if (!init_user_ids(owner, domain.c_str()))
 			{
 				dprintf(D_ALWAYS, "init_user_ids() failed!\n");
 			}
@@ -345,13 +355,13 @@ int BaseShadow::cdToIwd() {
 
 
 void
-BaseShadow::shutDownFast( int reason ) {
+BaseShadow::shutDownFast(int reason, const char* reason_str, int reason_code, int reason_subcode) {
 	m_force_fast_starter_shutdown = true;
-	shutDown( reason );
+	shutDown(reason, reason_str, reason_code, reason_subcode);
 }
 
 void
-BaseShadow::shutDown( int reason ) 
+BaseShadow::shutDown(int reason, const char* reason_str, int reason_code, int reason_subcode) 
 {
 		// exit now if there is no job ad
 	if ( !getJobAd() ) {
@@ -359,16 +369,11 @@ BaseShadow::shutDown( int reason )
 	}
 		//Attempt to write Job ad to epoch file
 		//If knob isn't set or there is no job ad the function will just log and return
-	writeJobEpochFile(getJobAd());
-		// if we are being called from the exception handler, return
-		// now to prevent infinite loop in case we call EXCEPT below.
-	if ( reason == JOB_EXCEPTION ) {
-		return;
-	}
 
 		// Only if the job is trying to leave the queue should we
 		// evaluate the user job policy...
 	if( reason == JOB_EXITED || reason == JOB_COREDUMPED ) {
+		writeJobEpochFile(getJobAd());
 		if( !waitingToUpdateSchedd() ) {
 			shadow_user_policy.checkAtExit();
 				// WARNING: 'this' may have been deleted by the time we get here!!!
@@ -377,7 +382,7 @@ BaseShadow::shutDown( int reason )
 	else {
 		// if we aren't trying to evaluate the user's policy, we just
 		// want to evict this job.
-		evictJob(reason, "");
+		evictJob(reason, reason_str, reason_code, reason_subcode);
 	}
 }
 
@@ -405,7 +410,6 @@ BaseShadow::reconnectFailed( const char* reason )
 	dprintf( D_ALWAYS, "Reconnect FAILED: %s\n", reason );
 	
 	logReconnectFailedEvent( reason );
-	writeJobEpochFile(getJobAd());
 
 		// if the shadow was born disconnected, exit with 
 		// JOB_RECONNECT_FAILED so the schedd can make 
@@ -414,15 +418,14 @@ BaseShadow::reconnectFailed( const char* reason )
 	if ( attemptingReconnectAtStartup ) {
 		dprintf(D_ALWAYS,"Exiting with JOB_RECONNECT_FAILED\n");
 		// does not return
-		DC_Exit( JOB_RECONNECT_FAILED );
+		evictJob(JOB_RECONNECT_FAILED, reason, CONDOR_HOLD_CODE::ReconnectFailed);
 	} else {
 		int exit_reason = getExitReason();
 		if (exit_reason == -1) {
 			exit_reason = JOB_SHOULD_REQUEUE;
 		}
-		dprintf(D_ALWAYS,"Exiting with %d\n", exit_reason);
 		// does not return
-		DC_Exit( exit_reason );
+		evictJob(exit_reason, reason, CONDOR_HOLD_CODE::ReconnectFailed);
 	}
 
 	// Should never get here....
@@ -430,20 +433,20 @@ BaseShadow::reconnectFailed( const char* reason )
 }
 
 std::string
-BaseShadow::improveHoldAttributes(const char* const orig_hold_reason, int & hold_reason_code, int & /*hold_reason_subcode*/ )
+BaseShadow::improveReasonAttributes(const char* orig_reason_str, int & reason_code, int & /*reason_subcode*/ )
 {
-	/* This method is invokved by the shadow when we are putting a job on hold, and allow us to
-	   change/edit the HoldReason attribute and HoldReason codes before the shadow sends the info
+	/* This method is invokved by the shadow when we are putting a job on hold or back to idle, and allow us to
+	   change/edit the Hold/EvictReason attribute and Hold/EvictReason codes before the shadow sends the info
 	   to the schedd.
-	   To change HoldReason string:  just set variable new_hold_reason - if it remains the empty string,
-	       then the orig_hold_reason will be used unmodified.
-	   To change HoldReasonCode or HoldReasonSubCode: just change the value of hold_reason_code and
-	       hold_reason_subcode respectively (not they are passed in by reference).
+	   To change Hold/EvictReason string:  just set variable new_reason_str - if it remains the empty string,
+	       then the orig_reason_str will be used unmodified.
+	   To change Hold/EvictReasonCode or Hold/EvictReasonSubCode: just change the value of reason_code and
+	       reason_subcode respectively (note they are passed in by reference).
 	*/
-	std::string new_hold_reason;
+	std::string new_reason_str;
 
 	/*
-	Improve hold reason/codes related to file transfer.  Here is a table of what
+	Improve reason/codes related to file transfer.  Here is a table of what
 	gets set by the file transfer object (from the starter), which is pretty
 	much useless to anyone who is not an expert:
 
@@ -459,20 +462,20 @@ BaseShadow::improveHoldAttributes(const char* const orig_hold_reason, int & hold
 		4. HoldReasonCode = 12 (DownloadFileError) if transfer of output files fail w/ error at AP
 		   Sample HoldReason = "Error from slot1@TODDS480S: STARTER at 127.0.0.1 failed to send file(s) to <127.0.0.1:50288>; SHADOW at 127.0.0.1 failed to write to file C:\condor\test\not_there\blah: (errno 2) No such file or directory"
 
-	Here we will rewrite the hold reason code to be TransferInputError or TransferOutputError, instead of
+	Here we will rewrite the reason code to be TransferInputError or TransferOutputError, instead of
 	the alsmost meaningless UploadFileError and DownloadFileError we get from the File Transfer object.
-	And try to improve the hold reason string as well, so people who shower have a chance understanding it.
+	And try to improve the reason string as well, so people who shower have a chance understanding it.
 	*/
-	if (hold_reason_code == FILETRANSFER_HOLD_CODE::DownloadFileError ||
-		hold_reason_code == FILETRANSFER_HOLD_CODE::UploadFileError)
+	if (reason_code == FILETRANSFER_HOLD_CODE::DownloadFileError ||
+		reason_code == FILETRANSFER_HOLD_CODE::UploadFileError)
 	{
 		bool transfer_input_error = !began_execution;
 		bool transfer_output_error = began_execution;
 		bool err_occurred_at_EP =
-			(transfer_input_error && hold_reason_code == FILETRANSFER_HOLD_CODE::DownloadFileError) ||
-			(transfer_output_error && hold_reason_code == FILETRANSFER_HOLD_CODE::UploadFileError);
+			(transfer_input_error && reason_code == FILETRANSFER_HOLD_CODE::DownloadFileError) ||
+			(transfer_output_error && reason_code == FILETRANSFER_HOLD_CODE::UploadFileError);
 
-		std::string old_reason = orig_hold_reason;
+		std::string old_reason = orig_reason_str;
 		size_t pos = std::string::npos;
 
 		// Try to parse out the name of the slot / EP involved
@@ -524,26 +527,26 @@ BaseShadow::improveHoldAttributes(const char* const orig_hold_reason, int & hold
 			}
 		}
 
-		// Currently, the hold_reason_code is set to either FILETRANSFER_HOLD_CODE::DownloadFileError
+		// Currently, the reason_code is set to either FILETRANSFER_HOLD_CODE::DownloadFileError
 		// or FILETRANSFER_HOLD_CODE::UploadFileError.  These are not very useful to users.
-		// So reset the hold_reason_code to something based on if the failure happened
+		// So reset the reason_code to something based on if the failure happened
 		// during transfer of input files or during transfer of output/checkpoint files.
 		if (transfer_input_error) {
-			hold_reason_code = CONDOR_HOLD_CODE::TransferInputError;
+			reason_code = CONDOR_HOLD_CODE::TransferInputError;
 		}
 		else {
-			hold_reason_code = CONDOR_HOLD_CODE::TransferOutputError;
+			reason_code = CONDOR_HOLD_CODE::TransferOutputError;
 		}
 
 		// Now create a more human-readable HoldReason string.
 		std::string EP_name = slot_name.empty() ? "the execution point" : ("execution point " + slot_name);
-		formatstr(new_hold_reason, "Transfer %s files failure at ",
+		formatstr(new_reason_str, "Transfer %s files failure at ",
 			transfer_input_error ? "input" : "output");
 		if (err_occurred_at_EP) {
 			// Error happened at the EP
-			new_hold_reason += EP_name;
+			new_reason_str += EP_name;
 			if (url_file_type.empty()) {
-				formatstr_cat(new_hold_reason, " while %s files %s access point %s",
+				formatstr_cat(new_reason_str, " while %s files %s access point %s",
 					transfer_input_error ? "receiving" : "sending",
 					transfer_input_error ? "from" : "to",
 					get_local_hostname().c_str());
@@ -551,20 +554,20 @@ BaseShadow::improveHoldAttributes(const char* const orig_hold_reason, int & hold
 		}
 		else {
 			// Error happened at the AP
-			formatstr_cat(new_hold_reason, "access point %s while %s files %s %s",
+			formatstr_cat(new_reason_str, "access point %s while %s files %s %s",
 				get_local_hostname().c_str(),
 				transfer_output_error ? "receiving" : "sending",
 				transfer_output_error ? "from" : "to",
 				EP_name.c_str());
 		}
 		if (!url_file_type.empty()) {
-			formatstr_cat(new_hold_reason, " using protocol %s", url_file_type.c_str());
+			formatstr_cat(new_reason_str, " using protocol %s", url_file_type.c_str());
 		}
-		formatstr_cat(new_hold_reason, ". Details: %s",
-			actual_error.empty() ? orig_hold_reason : actual_error.c_str());
+		formatstr_cat(new_reason_str, ". Details: %s",
+			actual_error.empty() ? orig_reason_str : actual_error.c_str());
 	}
 
-	return new_hold_reason;
+	return new_reason_str;
 }
 
 void
@@ -577,15 +580,6 @@ BaseShadow::holdJob( const char* reason, int hold_reason_code, int hold_reason_s
 		DC_Exit( JOB_SHOULD_HOLD );
 	}
 
-	// Note: improveHoldAttributescan change the hold_reason_code and subcode,
-	// and will pass back a potentially improved hold reason string.
-	std::string improved_hold_reason =
-		improveHoldAttributes(reason, hold_reason_code, hold_reason_subcode);
-	// If we have an improved hold reason string, use it instead.
-	if (!improved_hold_reason.empty()) {
-		reason = improved_hold_reason.c_str();
-	}
-
 	dprintf(D_ALWAYS, "Job %d.%d going into Hold state (code %d,%d): %s\n",
 		getCluster(), getProc(), hold_reason_code, hold_reason_subcode, reason);
 
@@ -895,7 +889,7 @@ BaseShadow::terminateJob( update_style_t kind ) // has a default argument of US_
 
 
 void
-BaseShadow::evictJob( int reason, const std::string &reasonStr)
+BaseShadow::evictJob( int exit_reason, const char* reason_str, int reason_code, int reason_subcode)
 {
 	std::string from_where;
 	std::string machine;
@@ -903,11 +897,36 @@ BaseShadow::evictJob( int reason, const std::string &reasonStr)
 	// If we previously delayed exiting to let the starter wrap up, then
 	// immediately try exiting now. None of the cleanup below here is
 	// appropriate in that case.
-	if ( exitDelayed( reason ) ) {
-		exitAfterEvictingJob( reason );
+	if ( exitDelayed( exit_reason ) ) {
+		exitAfterEvictingJob( exit_reason );
 		return;
 	}
 
+	if (reason_str == nullptr || *reason_str == '\0') {
+		switch(exit_reason) {
+		case JOB_SHOULD_REQUEUE:
+			if (!reason_str) reason_str = "Unspecified job interruption";
+			reason_code = CONDOR_HOLD_CODE::JobShouldRequeue;
+			break;
+		case JOB_NOT_STARTED:
+			if (!reason_str) reason_str = "Problem preparing job to run";
+			reason_code = CONDOR_HOLD_CODE::JobNotStarted;
+			break;
+		default:
+			// Other exit reasons don't need a reason
+			reason_str = "";
+		}
+	}
+
+	// Note: improveReasonAttributescan change the reason_code and subcode,
+	// and will pass back a potentially improved reason string.
+	std::string improved_reason_str =
+		improveReasonAttributes(reason_str, reason_code, reason_subcode);
+	// If we have an improved reason string, use it instead.
+	if (!improved_reason_str.empty()) {
+		reason_str = improved_reason_str.c_str();
+	}
+
 	if( getMachineName(machine) ) {
 		formatstr(from_where, " from %s" ,machine.c_str());
 	}
@@ -916,17 +935,24 @@ BaseShadow::evictJob( int reason, const std::string &reasonStr)
 
 	if( ! jobAd ) {
 		dprintf( D_ALWAYS, "In evictJob() w/ NULL JobAd!\n" );
-		DC_Exit( reason );
+		DC_Exit( exit_reason );
 	}
 
+		// record details about this vacate into the job ad
+	jobAd->Assign(ATTR_LAST_VACATE_TIME, time(nullptr));
+	jobAd->Assign(ATTR_VACATE_REASON, reason_str);
+	jobAd->Assign(ATTR_VACATE_REASON_CODE, reason_code);
+	jobAd->Assign(ATTR_VACATE_REASON_SUBCODE, reason_subcode);
+
+	writeJobEpochFile(getJobAd());
+
+	if (exit_reason != JOB_RECONNECT_FAILED) {
 		// cleanup this shadow (kill starters, etc)
-	cleanUp( jobWantsGracefulRemoval() );
+		cleanUp( jobWantsGracefulRemoval() );
 
 		// write stuff to user log:
-	logEvictEvent( reason, reasonStr);
-
-		// record the time we were vacated into the job ad 
-	jobAd->Assign( ATTR_LAST_VACATE_TIME, time(nullptr) );
+		logEvictEvent( exit_reason, reason_str);
+	}
 
 		// update the job ad in the queue with some important final
 		// attributes so we know what happened to the job when using
@@ -936,7 +962,7 @@ BaseShadow::evictJob( int reason, const std::string &reasonStr)
 		dprintf( D_ALWAYS, "Failed to update job queue!\n" );
 	}
 
-	exitAfterEvictingJob( reason );
+	exitAfterEvictingJob( exit_reason );
 }
 
 
@@ -1200,9 +1226,7 @@ BaseShadow::logEvictEvent( int exitReason, const std::string &reasonStr )
 	}
 
 	JobEvictedEvent event;
-	if (!reasonStr.empty()) {
-		event.reason = reasonStr;
-	}
+	event.setReason(reasonStr);
 	event.checkpointed = (exitReason == JOB_CKPTED);
 	
 		// TODO: fill in local rusage
@@ -1254,7 +1278,7 @@ BaseShadow::logRequeueEvent( const char* reason )
 	}
 
 	if( reason ) {
-		event.reason = reason;
+		event.setReason(reason);
 	}
 
 		// TODO: fill in local rusage
@@ -1313,12 +1337,10 @@ BaseShadow::checkSwap( void )
 
 // Note: log_except is static
 void
-BaseShadow::log_except(const char *msg)
+BaseShadow::log_except(const char *msg_str)
 {
-	if(!msg) msg = "";
-
 	if ( BaseShadow::myshadow_ptr == NULL ) {
-		::dprintf (D_ALWAYS, "Unable to log ULOG_SHADOW_EXCEPTION event (no Shadow object): %s\n", msg);
+		::dprintf (D_ALWAYS, "Unable to log ULOG_SHADOW_EXCEPTION event (no Shadow object): %s\n", msg_str ? msg_str : "");
 		return;
 	}
 
@@ -1326,8 +1348,8 @@ BaseShadow::log_except(const char *msg)
 	ShadowExceptionEvent event;
 	bool exception_already_logged = false;
 
-	snprintf(event.message, sizeof(event.message), "%s", msg);
-	event.message[sizeof(event.message)-1] = '\0';
+	// setMessage will convert any \n and \r in the message to | and space respectively
+	if (msg_str && msg_str[0]) { event.setMessage(msg_str); }
 
 	BaseShadow *shadow = BaseShadow::myshadow_ptr;
 
@@ -1342,9 +1364,11 @@ BaseShadow::log_except(const char *msg)
 		event.began_execution = TRUE;
 	}
 
+	Shadow->getJobAd()->Assign(ATTR_JOB_LAST_SHADOW_EXCEPTION, event.getMessage());
+	Shadow->updateJobInQueue(U_STATUS);
 	if (!exception_already_logged && !shadow->uLog.writeEventNoFsync (&event,shadow->jobAd))
 	{
-		::dprintf (D_ALWAYS, "Failed to log ULOG_SHADOW_EXCEPTION event: %s\n", msg);
+		::dprintf (D_ALWAYS, "Failed to log ULOG_SHADOW_EXCEPTION event: %s\n", event.getMessage());
 	}
 }
 
@@ -1563,12 +1587,22 @@ extern BaseShadow *Shadow;
 int
 display_dprintf_header(char **buf,int *bufpos,int *buflen)
 {
+	constexpr int cchpid = 10 * (sizeof(pid_t)/4); // 10 chars for 32 bit pids, 19 chars for 64 bit pids
+	static char pidbuf[cchpid+2 +1 + cchpid+2 +2] = {0}; // room for "()>()" + cchpid digits for each pid plus trailing \0
 	static pid_t mypid = 0;
 	int mycluster = -1;
 	int myproc = -1;
 
-	if (!mypid) {
-		mypid = daemonCore->getpid();
+	// show the shadow pid when we first start up
+	// then if we fork show the forked pid also
+	if (daemonCore) {
+		pid_t tpid = daemonCore->getpid();
+		if (!pidbuf[0]) {
+			mypid = tpid;
+			snprintf(pidbuf, sizeof(pidbuf)-1, "(%d)", mypid);
+		} else if (tpid != mypid) {
+			snprintf(pidbuf, sizeof(pidbuf)-1, "(%d)>(%d)", mypid, tpid);
+		}
 	}
 
 	if (Shadow) {
@@ -1577,9 +1611,9 @@ display_dprintf_header(char **buf,int *bufpos,int *buflen)
 	}
 
 	if ( mycluster != -1 ) {
-		return sprintf_realloc( buf, bufpos, buflen, "(%d.%d) (%ld): ", mycluster, myproc, (long)mypid );
+		return sprintf_realloc( buf, bufpos, buflen, "(%d.%d) %s: ", mycluster, myproc, pidbuf );
 	} else {
-		return sprintf_realloc( buf, bufpos, buflen, "(?.?) (%ld): ", (long)mypid );
+		return sprintf_realloc( buf, bufpos, buflen, "(?.?) %s: ", pidbuf );
 	}
 
 	return 0;
