@@ -111,7 +111,7 @@ RemoteResource::RemoteResource( BaseShadow *shad )
 
 	std::string prefix;
 	param(prefix, "CHIRP_DELAYED_UPDATE_PREFIX");
-	m_delayed_update_prefix.initializeFromString(prefix.c_str());
+	m_delayed_update_prefix = split(prefix);
 
 	param_and_insert_attrs("PROTECTED_JOB_ATTRS", m_unsettable_attrs);
 	param_and_insert_attrs("SYSTEM_PROTECTED_JOB_ATTRS", m_unsettable_attrs);
@@ -478,7 +478,7 @@ RemoteResource::attemptShutdown()
 	abortFileTransfer();
 
 		// we call our shadow's shutdown method:
-	shadow->shutDown( exit_reason );
+	shadow->shutDown( exit_reason, "" );
 }
 
 int
@@ -1213,7 +1213,7 @@ RemoteResource::updateFromStarter( ClassAd* update_ad )
 	// expression, then just copy the expression into the job ad.
 	classad::ExprTree * tree = update_ad->Lookup(ATTR_MEMORY_USAGE);
 	if (tree) {
-		if (tree->GetKind() != ExprTree::LITERAL_NODE) {
+		if (dynamic_cast<classad::Literal *>(tree) == nullptr) {
 				// Copy the exression over
 			tree = tree->Copy();
 			jobAd->Insert(ATTR_MEMORY_USAGE, tree);
@@ -2198,7 +2198,7 @@ RemoteResource::initFileTransfer()
 	int r = filetrans.Init( jobAd, false, PRIV_USER, spool_time != 0 );
 	if (r == 0) {
 		// filetransfer Init failed
-		EXCEPT( "RemoteResource::initFileTransfer  Init failed\n");
+		EXCEPT( "RemoteResource::initFileTransfer  Init failed");
 	}
 
 	filetrans.RegisterCallback(
@@ -2704,7 +2704,7 @@ RemoteResource::allowRemoteWriteAttributeAccess( const std::string &name )
 	bool response = m_want_chirp || m_want_remote_updates;
 	if (!response && m_want_delayed)
 	{
-		response = m_delayed_update_prefix.contains_anycase_withwildcard(name.c_str());
+		response = contains_anycase_withwildcard(m_delayed_update_prefix, name);
 	}
 
 	// Since this function is called to see if a user job is allowed to update
