@@ -27,6 +27,7 @@
 #include "metric_units.h"
 #include "ShadowHookMgr.h"
 #include "store_cred.h"
+#include "condor_holdcodes.h"
 
 extern "C" char* d_format_time(double);
 
@@ -125,8 +126,11 @@ UniShadow::spawnFinish()
 	hookTimerCancel();
 	if( ! remRes->activateClaim() ) {
 			// we're screwed, give up:
-		shutDown( JOB_NOT_STARTED );
+		shutDown(JOB_NOT_STARTED, "Failed to activate claim", CONDOR_HOLD_CODE::FailedToActivateClaim);
 	}
+	// Start the timer for the periodic user job policy
+	shadow_user_policy.startTimer();
+
 }
 
 
@@ -141,7 +145,7 @@ UniShadow::spawn()
 		if (rval == -1) {
 			dprintf(D_ALWAYS, "Prepare job hook has failed.  Will shutdown job.\n");
 			BaseShadow::log_except("Submit-side job hook execution failed");
-			shutDown(JOB_NOT_STARTED);
+			shutDown(JOB_NOT_STARTED, "Shadow prepare hook failed");
 		} else if (rval == 0) {
 			dprintf(D_FULLDEBUG, "No prepare job hook to run - activating job immediately.\n");
 			spawnFinish();
@@ -152,7 +156,7 @@ UniShadow::spawn()
 				"hookTimeout",
 				this);
 		} else {
-			EXCEPT("Hook manager returned an invalid code\n");
+			EXCEPT("Hook manager returned an invalid code");
 		}
 	}
 }
@@ -163,7 +167,7 @@ UniShadow::hookTimeout( int /* timerID */ )
 {
 	dprintf(D_ERROR, "Timed out waiting for a hook to exit\n");
 	BaseShadow::log_except("Submit-side job hook execution timed out");
-	shutDown(JOB_NOT_STARTED);
+	shutDown(JOB_NOT_STARTED, "Shadow prepare hook timed out");
 }
 
 
@@ -486,6 +490,8 @@ UniShadow::resourceReconnected( RemoteResource* rr )
 		if ( job_execute_date >= claim_start_date ) {
 			began_execution = true;
 		}
+		// Start the timer for the periodic user job policy
+		shadow_user_policy.startTimer();
 	}
 
 		// Since our reconnect worked, clear attemptingReconnectAtStartup
@@ -522,12 +528,7 @@ UniShadow::resourceReconnected( RemoteResource* rr )
 		requestJobRemoval();
 	}
 
-		// If we know the job is already executing, ensure the timers
-		// that are supposed to start then are running.
 	if (began_execution) {
-			// Start the timer for the periodic user job policy
-		shadow_user_policy.startTimer();
-
 			// Start the timer for updating the job queue for this job
 		startQueueUpdateTimer();
 	}
@@ -538,7 +539,7 @@ void
 UniShadow::logDisconnectedEvent( const char* reason )
 {
 	JobDisconnectedEvent event;
-	event.disconnect_reason = reason;
+	if (reason) { event.setDisconnectReason(reason); }
 
 	DCStartd* dc_startd = remRes->getDCStartd();
 	if( ! dc_startd ) {
@@ -582,7 +583,7 @@ UniShadow::logReconnectFailedEvent( const char* reason )
 {
 	JobReconnectFailedEvent event;
 
-	event.reason = reason;
+	if (reason) { event.setReason(reason); }
 
 	DCStartd* dc_startd = remRes->getDCStartd();
 	if( ! dc_startd ) {
