@@ -24,6 +24,7 @@
 #include <math.h>
 #include "filesystem_remap.h"
 #include "classad_helpers.h" // for cleanStringForUseAsAttr
+#include "../condor_sysapi/sysapi.h"
 
 #include <set>
 #include <algorithm>
@@ -37,7 +38,7 @@
 #include "docker-api.h"
 
 MachAttributes::MachAttributes()
-   : m_user_specified(NULL, ";"), m_user_settings_init(false), m_named_chroot()
+   : m_user_settings_init(false), m_named_chroot()
 {
 	m_mips = -1;
 	m_kflops = -1;
@@ -160,19 +161,16 @@ MachAttributes::~MachAttributes()
 	if( m_utsname_version ) free( m_utsname_version );
 	if( m_utsname_machine ) free( m_utsname_machine );
 
-    AttribValue *val = NULL;
-    m_lst_dynamic.Rewind();
-    while ((val = m_lst_dynamic.Next()) ) {
+	for (auto *val : m_lst_dynamic) {
        if (val) free (val);
-       m_lst_dynamic.DeleteCurrent();
-    }
+	}
+	m_lst_dynamic.clear();
 
-    m_lst_static.Rewind();
-    while ((val = m_lst_static.Next())) {
+	for (auto *val: m_lst_static) {
        if (val) free (val);
-       m_lst_static.DeleteCurrent();
     }
-    m_user_specified.clearAll();
+	m_lst_static.clear();
+
 #if defined(WIN32)
 	if( m_local_credd ) free( m_local_credd );
     if( m_dot_Net_Versions ) free ( m_dot_Net_Versions );
@@ -186,47 +184,41 @@ MachAttributes::init_user_settings()
 {
 	m_user_settings_init = true;
 
-	AttribValue *val = NULL;
-	m_lst_dynamic.Rewind();
-	while ((val = m_lst_dynamic.Next()))
-    {
+	for (auto *val : m_lst_dynamic) {
         if (val) free (val);
-	    m_lst_dynamic.DeleteCurrent();
 	}
+	m_lst_dynamic.clear();
 
-	m_lst_static.Rewind();
-	while ((val = m_lst_static.Next()))
-    {
+	for (auto *val: m_lst_static) {
 	    if (val) free (val);
-	    m_lst_static.DeleteCurrent();
 	}
+	m_lst_static.clear();
 
-	m_user_specified.clearAll();
+	m_user_specified.clear();
 
 #ifdef WIN32
 	char * pszParam = NULL;
 	pszParam = param("STARTD_PUBLISH_WINREG");
 	if (pszParam)
     {
-		m_user_specified.initializeFromString(pszParam);
+		m_user_specified = split(pszParam, ";");
 		free(pszParam);
 	}
 #endif
 
-	m_user_specified.rewind();
-	while(char * pszItem = m_user_specified.next())
+	for (const auto& pszItem: m_user_specified)
     {
 		// if the reg_item is of the form attr_name=reg_path;
 		// then skip over the attr_name and '=' and trailing
 		// whitespace.  But if the = is after the first \, then
 		// it's a part of the reg_path, so ignore it.
 		//
-		const char * pkey = strchr(pszItem, '=');
-		const char * pbs  = strchr(pszItem, '\\');
+		const char * pkey = strchr(pszItem.c_str(), '=');
+		const char * pbs  = strchr(pszItem.c_str(), '\\');
 		if (pkey && ( ! pbs || pkey < pbs)) {
 			++pkey; // skip the '='
 		} else {
-			pkey = pszItem;
+			pkey = pszItem.c_str();
 		}
 
 		// skip any leading whitespace in the key
@@ -234,7 +226,7 @@ MachAttributes::init_user_settings()
 			++pkey;
 
        #ifdef WIN32
-		char * pszAttr = generate_reg_key_attr_name("WINREG_", pszItem);
+		char * pszAttr = generate_reg_key_attr_name("WINREG_", pszItem.c_str());
 
 		// if the keyname begins with 32 or 64, use that to designate either
 		// the WOW64 or WOW32 view of the registry, but don't pass the
@@ -261,7 +253,7 @@ MachAttributes::init_user_settings()
 			//
             AttribValue * pav = add_WinPerf_Query(pszAttr, pkey+ixStart+1);
             if (pav)
-		        m_lst_dynamic.Append(pav);
+		        m_lst_dynamic.emplace_back(pav);
 		}
         else
         {
@@ -273,7 +265,7 @@ MachAttributes::init_user_settings()
             if (pav)
             {
                 pav->pquery = (void*)pkey;
-				m_lst_static.Append(pav);
+				m_lst_static.emplace_back(pav);
 			}
 		}
 
@@ -524,9 +516,7 @@ MachAttributes::compute_for_policy()
         update_all_WinPerf_results();
 #endif
 
-        AttribValue *pav = NULL;
-        m_lst_dynamic.Rewind();
-        while ((pav = m_lst_dynamic.Next()) ) {
+		for (auto *pav : m_lst_dynamic) {
            if (pav) {
              #ifdef WIN32
               if ( ! update_WinPerf_Value(pav))
@@ -943,9 +933,9 @@ bool MachAttributes::ComputeDevProps(
 // fungable resource, or a list of ids of non-fungable resources.
 // if res_value contains a list, then ids is set on exit from this function
 // otherwise, ids empty.
-static double parse_user_resource_config(const char * tag, const char * res_value, StringList & ids)
+static double parse_user_resource_config(const char * tag, const char * res_value, std::set<std::string> & ids)
 {
-	ids.clearAll();
+	ids.clear();
 	double num = 0;
 	char * pend = NULL;
 	num = strtod(res_value, &pend);
@@ -959,8 +949,10 @@ static double parse_user_resource_config(const char * tag, const char * res_valu
 			is_simple_double = true;
 		} else {
 			// not a simple double, so treat it as a list of id's, the number of resources is the number of ids
-			ids.initializeFromString(res_value);
-			num = ids.number();
+			for (const auto& item: StringTokenIterator(res_value)) {
+				ids.insert(item);
+			}
+			num = ids.size();
 		}
 	}
 
@@ -997,7 +989,7 @@ double MachAttributes::init_machine_resource_from_script(const char * tag, const
 			classad::Value value;
 			std::string attr(ATTR_OFFLINE_PREFIX); attr += tag;
 			std::string res_value;
-			StringList offline_ids;
+			std::set<std::string> offline_ids;
 			if (ad.LookupString(attr,res_value)) {
 				offline = parse_user_resource_config(tag, res_value.c_str(), offline_ids);
 			} else {
@@ -1009,13 +1001,12 @@ double MachAttributes::init_machine_resource_from_script(const char * tag, const
 
 			attr = ATTR_DETECTED_PREFIX; attr += tag;
 			if (ad.LookupString(attr,res_value)) {
-				StringList ids;
+				std::set<std::string> ids;
 				quantity = parse_user_resource_config(tag, res_value.c_str(), ids);
-				if ( ! ids.isEmpty()) {
+				if ( ! ids.empty()) {
 					offline = 0; // we only want to count ids that match the detected list
-					ids.rewind();
-					while (const char* id = ids.next()) {
-						if (offline_ids.contains(id)) {
+					for (const auto& id: ids) {
+						if (offline_ids.count(id)) {
 							unique_ids.insert(id);
 							this->m_machres_offline_devIds_map[tag].emplace_back(id);
 							++offline;
@@ -1044,7 +1035,7 @@ double MachAttributes::init_machine_resource_from_script(const char * tag, const
 				// then remove it from the nested ad collection so we don't see it when we iterate
 				auto common_it = nested.find("common");
 				if (common_it != nested.end()) {
-					for (auto id : unique_ids) {
+					for (const auto& id : unique_ids) {
 						m_machres_nft_map[tag].props[id].Update(*common_it->second);
 					}
 					ad.Delete(common_it->first);
@@ -1105,7 +1096,7 @@ bool MachAttributes::init_machine_resource(MachAttributes * pme, HASHITER & it)
 			num = pme->init_machine_resource_from_script(tag, res_value);
 		}
 	} else {
-		StringList offline_ids;
+		std::set<std::string> offline_ids;
 		std::string off_name("OFFLINE_"); off_name += name;
 		std::string my_value;
 		if (param(my_value, off_name.c_str()) && !my_value.empty()) {
@@ -1113,13 +1104,12 @@ bool MachAttributes::init_machine_resource(MachAttributes * pme, HASHITER & it)
 		}
 
 		// if the param value parses as a double, then we can handle it simply
-		StringList ids;
+		std::set<std::string> ids;
 		num = parse_user_resource_config(tag, res_value, ids);
-		if ( ! ids.isEmpty()) {
+		if ( ! ids.empty()) {
 			offline = 0; // we only want to count ids that match the detected list
-			ids.rewind();
-			while (const char* id = ids.next()) {
-				if (offline_ids.contains(id)) {
+			for (const auto& id: ids) {
+				if (offline_ids.count(id)) {
 					pme->m_machres_offline_devIds_map[tag].emplace_back(id);
 					++offline;
 				} else {
@@ -1171,30 +1161,30 @@ void MachAttributes::init_machine_resources() {
 	const int iter_options = HASHITER_NO_DEFAULTS; // we can speed up iteration if there are no machine resources in the defaults table.
 	foreach_param_matching(re, iter_options, (bool(*)(void*,HASHITER&))MachAttributes::init_machine_resource, this);
 
-	StringList allowed;
+	std::vector<std::string> allowed;
 	char* allowed_names = param("MACHINE_RESOURCE_NAMES");
-	if (allowed_names && allowed_names[0]) {
+	if (allowed_names) {
 		// if admin defined MACHINE_RESOURCE_NAMES, then use this list
 		// as the source of expected custom machine resources
-		allowed.initializeFromString(allowed_names);
+		allowed = split(allowed_names);
 		free(allowed_names);
 	}
 
-	StringList disallowed("CPU CPUS DISK SWAP MEM MEMORY RAM");
+	std::vector<std::string> disallowed = {"CPU", "CPUS", "DISK", "SWAP", "MEM", "MEMORY", "RAM"};
 
 	std::vector<const char *> tags_to_erase;
-	for (slotres_map_t::iterator it(m_machres_map.begin());  it != m_machres_map.end();  ++it) {
-		const char * tag = it->first.c_str();
-		if ( ! allowed.isEmpty() && ! allowed.contains_anycase(tag)) {
+	for (auto & it : m_machres_map) {
+		const char * tag = it.first.c_str();
+		if ( ! allowed.empty() && ! contains_anycase(allowed, tag)) {
 			dprintf(D_ALWAYS, "Local machine resource %s is not in MACHINE_RESOURCE_NAMES so it will have no effect\n", tag);
 			tags_to_erase.emplace_back(tag);
 			continue;
 		}
-		if ( ! disallowed.isEmpty() && disallowed.contains_anycase(tag)) {
+		if ( ! disallowed.empty() && contains_anycase(disallowed, tag)) {
 			EXCEPT("fatal error - MACHINE_RESOURCE_%s is invalid, '%s' is a reserved resource name", tag, tag);
 			continue;
 		}
-		dprintf(D_ALWAYS, "Local machine resource %s = %g\n", tag, it->second);
+		dprintf(D_ALWAYS, "Local machine resource %s = %g\n", tag, it.second);
 	}
 	for( auto i = tags_to_erase.begin(); i != tags_to_erase.end(); ++i ) {
 		m_machres_map.erase(*i);
@@ -1367,8 +1357,7 @@ MachAttributes::publish_static(ClassAd* cp)
 	// publish values from the window's registry as specified
 	// in the STARTD_PUBLISH_WINREG param.
 	//
-	m_lst_static.Rewind();
-	while (AttribValue *pav = m_lst_static.Next()) {
+	for (auto *pav: m_lst_static) {
 		if (pav) pav->AssignToClassAd(cp);
 	}
 
@@ -1388,23 +1377,23 @@ MachAttributes::publish_static(ClassAd* cp)
 
 	std::string machine_resources = "Cpus Memory Disk Swap";
 	// publish any local resources
-	for (slotres_map_t::iterator j(m_machres_map.begin());  j != m_machres_map.end();  ++j) {
-		const char * rname = j->first.c_str();
+	for (auto & j : m_machres_map) {
+		const char * rname = j.first.c_str();
 		std::string attr(ATTR_DETECTED_PREFIX); attr += rname;
-		double ipart, fpart = modf(j->second, &ipart);
+		double ipart, fpart = modf(j.second, &ipart);
 		if (fpart >= 0.0 && fpart <= 0.0) {
 			cp->Assign(attr, (long long)ipart);
 		} else {
-			cp->Assign(attr, j->second);
+			cp->Assign(attr, j.second);
 		}
 		attr = ATTR_TOTAL_PREFIX; attr += rname;
 		if (fpart >= 0.0 && fpart <= 0.0) {
 			cp->Assign(attr, (long long)ipart);
 		} else {
-			cp->Assign(attr, j->second);
+			cp->Assign(attr, j.second);
 		}
 		machine_resources += " ";
-		machine_resources += j->first;
+		machine_resources += j.first;
 	}
 	cp->Assign(ATTR_MACHINE_RESOURCES, machine_resources);
 
@@ -1416,10 +1405,7 @@ MachAttributes::publish_static(ClassAd* cp)
 	// Advertise Docker Volumes
 	char *dockerVolumes = param("DOCKER_VOLUMES");
 	if (dockerVolumes) {
-		StringList vl(dockerVolumes);
-		vl.rewind();
-		char *volume = 0;
-		while ((volume = vl.next())) {
+		for (const auto& volume: StringTokenIterator(dockerVolumes)) {
 			std::string attrName = "HasDockerVolume";
 			attrName += volume;
 			cp->Assign(attrName, true);
@@ -1608,8 +1594,7 @@ MachAttributes::publish_common_dynamic(ClassAd* cp, bool global /*=false*/)
 	cp->Assign( ATTR_CLOCK_MIN, m_clock_min );
 	cp->Assign( ATTR_CLOCK_DAY, m_clock_day );
 
-	m_lst_dynamic.Rewind();
-	while (AttribValue *pav = m_lst_dynamic.Next() ) {
+	for (auto *pav : m_lst_dynamic) {
 		if (pav) pav->AssignToClassAd(cp);
 	}
 }
@@ -1618,12 +1603,12 @@ void
 MachAttributes::publish_slot_dynamic(ClassAd* cp, int slot_id, int slot_subid, bool slot_is_bk, const std::string & res_conflict)
 {
 	// the global resource conflicts are determined elsewhere and passed in here
-	// we we add the NFR conflicts and the publish
+	// we we add the NFR conflicts and then publish
 	std::string conflict = res_conflict;
 
 	// publish "Available" custom resources and resource properties
 	// for use by the evalInEachContext matchmaking function
-	// this loop also detects primary/backfill resource conflcits for non-fungible resources
+	// this loop also detects primary/backfill resource conflicts for non-fungible resources
 	for (auto &[restag, nft] : m_machres_nft_map) {
 		auto & offline_ids(m_machres_runtime_offline_ids_map[restag]);
 
@@ -1801,7 +1786,7 @@ MachAttributes::credd_test()
 }
 #endif
 
-CpuAttributes::CpuAttributes( MachAttributes* map_arg, 
+CpuAttributes::CpuAttributes(
 							  unsigned int slot_type,
 							  double num_cpus_arg,
 							  int num_phys_mem,
@@ -1812,7 +1797,6 @@ CpuAttributes::CpuAttributes( MachAttributes* map_arg,
 							  const std::string &execute_dir,
 							  const std::string &execute_partition_id )
 {
-	map = map_arg;
 	c_type_id = slot_type;
 	c_num_slot_cpus = c_num_cpus = num_cpus_arg;
 	c_allow_fractional_cpus = num_cpus_arg > 0 && num_cpus_arg < 0.9;
@@ -1843,7 +1827,7 @@ CpuAttributes::attach( Resource* res_ip )
 
 
 bool
-CpuAttributes::set_total_disk(long long total, bool refresh) {
+CpuAttributes::set_total_disk(long long total, bool refresh, VolumeManager * volman) {
 		// if input total is < 0, that means to figure it out
 	if (total > 0) {
 		bool changed = total != c_total_disk;
@@ -1855,20 +1839,18 @@ CpuAttributes::set_total_disk(long long total, bool refresh) {
 	}
 		// refresh disk if the flag was passed in, or we do not yet have a value
 	if (refresh) {
-#ifdef LINUX
 		CondorError err;
 		uint64_t used_bytes, total_bytes;
-		if (m_volume_mgr) {
-			if (m_volume_mgr->GetPoolSize(used_bytes, total_bytes, err)) {
+		if (volman && volman->is_enabled()) {
+			if (volman->GetPoolSize(used_bytes, total_bytes, err)) {
 				c_total_disk = total_bytes/1024 - sysapi_reserve_for_fs();
 				c_total_disk = (c_total_disk < 0) ? 0 : c_total_disk;
 				dprintf(D_FULLDEBUG, "Used volume manager to get total logical size of %llu\n", c_total_disk);
 				return true;
 			} else {
-				dprintf(D_FULLDEBUG, "Failure to get pool size: %s\n", err.getFullText().c_str());
+				dprintf(D_ERROR, "Failed to get LVM pool size: %s\n", err.getFullText().c_str());
 			}
 		}
-#endif // LINUX
 		c_total_disk = sysapi_disk_space(executeDir());
 		return true;
 	}
@@ -1876,7 +1858,7 @@ CpuAttributes::set_total_disk(long long total, bool refresh) {
 }
 
 bool
-CpuAttributes::bind_DevIds(int slot_id, int slot_sub_id, bool backfill_slot, bool abort_on_fail) // bind non-fungable resource ids to a slot
+CpuAttributes::bind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id, bool backfill_slot, bool abort_on_fail) // bind non-fungable resource ids to a slot
 {
 	if ( ! map)
 		return true;
@@ -1896,36 +1878,36 @@ CpuAttributes::bind_DevIds(int slot_id, int slot_sub_id, bool backfill_slot, boo
 		}
 	}
 
-	for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
+	for (auto & j : c_slotres_map) {
 		// TODO: handle fractional assigned custome resources?
-		int cAssigned = int(j->second);
+		int cAssigned = int(j.second);
 
 		// if this resource already has bound ids, don't bind again.
-		slotres_devIds_map_t::const_iterator m(c_slotres_ids_map.find(j->first));
+		slotres_devIds_map_t::const_iterator m(c_slotres_ids_map.find(j.first));
 		if (m != c_slotres_ids_map.end() && cAssigned == (int)m->second.size())
 			continue;
 
 		const char * request = nullptr;
-		slotres_constraint_map_t::const_iterator req(c_slotres_constraint_map.find(j->first));
+		slotres_constraint_map_t::const_iterator req(c_slotres_constraint_map.find(j.first));
 		if (req != c_slotres_constraint_map.end()) { request = req->second.c_str(); }
 
-		::dprintf(D_ALWAYS, "bind %s DevIds tag=%s contraint=%s\n", name, j->first.c_str(), request ? request : "");
+		::dprintf(D_ALWAYS, "bind %s DevIds tag=%s contraint=%s\n", name, j.first.c_str(), request ? request : "");
 
-		if (map->machres_devIds().count(j->first)) {
+		if (map->machres_devIds().count(j.first)) {
 			int cAllocated = 0;
 			for (int ii = 0; ii < cAssigned; ++ii) {
-				const char * id = map->AllocateDevId(j->first, request, slot_id, slot_sub_id, backfill_slot);
+				const char * id = map->AllocateDevId(j.first, request, slot_id, slot_sub_id, backfill_slot);
 				if (id) {
 					++cAllocated;
-					c_slotres_ids_map[j->first].push_back(id);
+					c_slotres_ids_map[j.first].push_back(id);
 					::dprintf(d_log_devids, "bind DevIds for %s bound %s %d\n",
-						name, id, (int)c_slotres_ids_map[j->first].size());
+						name, id, (int)c_slotres_ids_map[j.first].size());
 				}
 			}
 			if (cAllocated < cAssigned) {
-				::dprintf(D_ALWAYS | D_BACKTRACE, "%s Failed to bind local resource '%s'\n", name, j->first.c_str());
+				::dprintf(D_ALWAYS | D_BACKTRACE, "%s Failed to bind local resource '%s'\n", name, j.first.c_str());
 				if (abort_on_fail) {
-					EXCEPT("Failed to bind local resource '%s'", j->first.c_str());
+					EXCEPT("Failed to bind local resource '%s'", j.first.c_str());
 				}
 				return false;
 			}
@@ -1934,8 +1916,8 @@ CpuAttributes::bind_DevIds(int slot_id, int slot_sub_id, bool backfill_slot, boo
 			// of making c_slot_res_ids_map['tag'] exist, which in turn results in the
 			// "Assigned<Tag>" slot attribute being defined rather than undefined.
 			if (cAllocated > 0) {
-				c_slotres_props_map[j->first].Clear();
-				map->ComputeDevProps(c_slotres_props_map[j->first], j->first, c_slotres_ids_map[j->first]);
+				c_slotres_props_map[j.first].Clear();
+				map->ComputeDevProps(c_slotres_props_map[j.first], j.first, c_slotres_ids_map[j.first]);
 			}
 		}
 	}
@@ -1953,7 +1935,7 @@ CpuAttributes::bind_DevIds(int slot_id, int slot_sub_id, bool backfill_slot, boo
 }
 
 void
-CpuAttributes::unbind_DevIds(int slot_id, int slot_sub_id) // release non-fungable resource ids
+CpuAttributes::unbind_DevIds(MachAttributes* map, int slot_id, int slot_sub_id) // release non-fungable resource ids
 {
 	if ( ! map) return;
 
@@ -1964,12 +1946,12 @@ CpuAttributes::unbind_DevIds(int slot_id, int slot_sub_id) // release non-fungab
 
 	if ( ! slot_sub_id) return;
 
-	for (slotres_map_t::iterator j(c_slotres_map.begin());  j != c_slotres_map.end();  ++j) {
-		slotres_devIds_map_t::const_iterator k(c_slotres_ids_map.find(j->first));
+	for (auto & j : c_slotres_map) {
+		slotres_devIds_map_t::const_iterator k(c_slotres_ids_map.find(j.first));
 		if (k != c_slotres_ids_map.end()) {
-			slotres_assigned_ids_t & ids = c_slotres_ids_map[j->first];
+			slotres_assigned_ids_t & ids = c_slotres_ids_map[j.first];
 			while ( ! ids.empty()) {
-				bool released = map->ReleaseDynamicDevId(j->first, ids.back().c_str(), slot_id, slot_sub_id);
+				bool released = map->ReleaseDynamicDevId(j.first, ids.back().c_str(), slot_id, slot_sub_id);
 				dprintf(released ? d_log_devids : D_ALWAYS, "ubind DevIds for slot%d.%d unbind %s %d %s\n",
 					slot_id, slot_sub_id, ids.back().c_str(), (int)ids.size(), released ? "OK" : "failed");
 				ids.pop_back();
@@ -1984,7 +1966,7 @@ CpuAttributes::unbind_DevIds(int slot_id, int slot_sub_id) // release non-fungab
 }
 
 void
-CpuAttributes::reconfig_DevIds(int slot_id, int slot_sub_id) // release non-fungable resource ids
+CpuAttributes::reconfig_DevIds(MachAttributes* map, int slot_id, int slot_sub_id) // release non-fungable resource ids
 {
 	if (! map) return;
 
@@ -2045,7 +2027,7 @@ CpuAttributes::publish_static(ClassAd* cp, const ResBag * inuse) const
 			int deduct = MAX(0, inuse->mem);
 			mem = MAX(0, mem - deduct);
 		}
-		cp->Assign( ATTR_MEMORY, c_phys_mem );
+		cp->Assign( ATTR_MEMORY, mem );
 		cp->Assign( ATTR_TOTAL_SLOT_MEMORY, c_slot_mem );
 		cp->Assign( ATTR_TOTAL_SLOT_DISK, c_slot_disk );
 
@@ -2101,10 +2083,10 @@ CpuAttributes::publish_static(ClassAd* cp, const ResBag * inuse) const
 }
 
 void
-CpuAttributes::compute_virt_mem()
+CpuAttributes::compute_virt_mem_share(double virt_mem)
 {
 	// Shared attributes that we only get a fraction of
-	double val = map->virt_mem();
+	double val = virt_mem;
 	if (!IS_AUTO_SHARE(c_virt_mem_fraction)) {
 		val *= c_virt_mem_fraction;
 	}
@@ -2210,7 +2192,7 @@ CpuAttributes::operator+=( CpuAttributes& rhs )
 	if (!IS_AUTO_SHARE(rhs.c_virt_mem_fraction) &&
 		!IS_AUTO_SHARE(c_virt_mem_fraction)) {
 		c_virt_mem_fraction += rhs.c_virt_mem_fraction;
-		c_virt_mem = map->virt_mem() * c_virt_mem_fraction;
+		c_virt_mem += rhs.c_virt_mem;   // not perfect, but close enough for now..
 	}
 
 	if (!IS_AUTO_SHARE(rhs.c_disk_fraction) &&
@@ -2251,6 +2233,24 @@ CpuAttributes::operator-=( CpuAttributes& rhs )
 	return *this;
 }
 
+const char * CpuAttributes::_slot_request::dump(std::string & buf) const
+{
+	buf.clear();
+	formatstr(buf, "Cpus=%f, Memory=%d, Disk=%f/1", num_cpus, num_phys_mem, disk_fraction);
+	if (virt_mem_fraction > 0) {
+		formatstr_cat(buf, ", Swap=%f/1", virt_mem_fraction);
+	}
+	for (auto &[tag,val] : slotres) {
+		formatstr_cat(buf, " ,%s=%f", tag.c_str(), val);
+		auto found = slotres_constr.find(tag);
+		if (found != slotres_constr.end()) {
+			formatstr_cat(buf, " : %s", found->second.c_str());
+		}
+	}
+	return buf.c_str();
+}
+
+
 ResBag&
 ResBag::operator+=(const CpuAttributes& rhs)
 {
@@ -2387,8 +2387,8 @@ AvailAttributes::AvailAttributes( MachAttributes* map ):
 	a_virt_mem_fraction = 1.0;
 	a_virt_mem_auto_count = 0;
     a_slotres_map = map->machres();
-    for (slotres_map_t::iterator j(a_slotres_map.begin());  j != a_slotres_map.end();  ++j) {
-        a_autocnt_map[j->first] = 0;
+    for (auto & j : a_slotres_map) {
+        a_autocnt_map[j.first] = 0;
     }
 }
 
@@ -2588,7 +2588,7 @@ AvailAttributes::cat_totals(std::string & buf, const char * execute_partition_id
 	formatstr_cat(buf, "Cpus: %d, Memory: %d, Swap: %.2f%%, Disk: %.2f%%",
 		a_num_cpus, a_phys_mem, 100*a_virt_mem_fraction,
 		100*partition.m_disk_fraction );
-	for (slotres_map_t::iterator j(a_slotres_map.begin());  j != a_slotres_map.end();  ++j) {
-		formatstr_cat(buf, ", %s: %d", j->first.c_str(), int(j->second));
+	for (auto & j : a_slotres_map) {
+		formatstr_cat(buf, ", %s: %d", j.first.c_str(), int(j.second));
 	}
 }
