@@ -22,6 +22,16 @@ while (-e $lockfile) {
 }
 
 
+# Um, actually...
+# Dagman doesn't remove us, a previous job in this DAG
+# has removed dagman, and the schedd some time later 
+# will remove us.  To fix common race conditions, create
+# a less common race condition, and sleep here, hoping
+# the schedd will actually remove us
+#
+
+sleep(1000);
+
 # We should get condor_rm'ed before we get to here...
 runcmd("touch $ARGV[0].finished");
 print "$ARGV[0] finished\n";
