@@ -32,7 +32,6 @@
 //#include "command_strings.h"
 //#include "enum_utils.h"
 #include "condor_version.h"
-#include "string_list.h"
 #include "ad_printmask.h"
 #include "internet.h"
 #include "compat_classad_util.h"
@@ -295,7 +294,7 @@ main( int argc, const char *argv[] )
 					continue;
 				}
 			}
-			if (cmd == EDIT_USERREC) {
+			if (strchr(argv[i], '=')) {
 				edit_args.push_back(argv[i]);
 			} else {
 				usernames.push_back(argv[i]);
@@ -379,7 +378,7 @@ main( int argc, const char *argv[] )
 				usage(stderr, my_name);
 				exit(1);
 			}
-			cmd = RESET_USERREC;
+			cmd = EDIT_USERREC;
 		}
 		else
 		if (is_dash_arg_colon_prefix (argv[i], "long", &pcolon, 1)) {
@@ -387,6 +386,10 @@ main( int argc, const char *argv[] )
 			if (pcolon) {
 				dash_long_format = parseAdsFileFormat(++pcolon, dash_long_format);
 			}
+			if ( ! prmask.IsEmpty()) {
+				fprintf( stderr, "Error: -long cannot be used with -format or -af\n" );
+				exit(1);
+			}
 		}
 		else
 		if (is_dash_arg_prefix (argv[i], "format", 1)) {
@@ -395,6 +398,10 @@ main( int argc, const char *argv[] )
 				fprintf( stderr, "Error: -format requires format and attribute parameters\n" );
 				exit( 1 );
 			}
+			if (dash_long) {
+				fprintf( stderr, "Error: -format and -long cannot be used together\n" );
+				exit( 1 );
+			}
 			prmask.registerFormatF( argv[i+1], argv[i+2], FormatOptionNoTruncate );
 			if ( ! IsValidClassAdExpression(argv[i+2], &attrs, NULL)) {
 				fprintf(stderr, "Error: -format arg '%s' is not a valid expression\n", argv[i+2]);
@@ -410,6 +417,10 @@ main( int argc, const char *argv[] )
 				fprintf( stderr, "Error: -autoformat requires at least one attribute parameter\n" );
 				exit(1);
 			}
+			if (dash_long) {
+				fprintf( stderr, "Error: -af and -long cannot be used together\n" );
+				exit(1);
+			}
 			int ixNext = parse_autoformat_args(argc, argv, i+1, pcolon, prmask, attrs, false);
 			if (ixNext < 0) {
 				fprintf(stderr, "Error: -autoformat arg '%s' is not a valid expression\n", argv[-ixNext]);
@@ -457,6 +468,12 @@ main( int argc, const char *argv[] )
 
 	if (!cmd) cmd = QUERY_USERREC_ADS;
 
+	if ( ! edit_args.empty() && cmd != EDIT_USERREC) {
+		fprintf(stderr, "<attr>=<expr> arguments only work with -edit");
+		usage(stderr, my_name);
+		exit(2);
+	}
+
 	if ((cmd == QUERY_USERREC_ADS) && ! dash_long && prmask.IsEmpty()) {
 		initOutputMask(prmask, 0, false);
 	}
@@ -509,11 +526,7 @@ main( int argc, const char *argv[] )
 			exit(1);
 		}
 
-		StringList white;
-		//StringList * whitelist = nullptr;
 		if ( ! attrs.empty()) {
-			for (auto attr : attrs) { white.insert(attr.c_str()); }
-			//whitelist = &white;
 			// make sure we ask for the key attribute when we query
 			attrs.insert(ATTR_USER);
 		}
@@ -536,30 +549,14 @@ main( int argc, const char *argv[] )
 		}
 
 		int connect_timeout = param_integer("Q_QUERY_TIMEOUT");
-	#if 1
 		rval = schedd.queryUsers(req_ad, process_ads, process_ads_data, connect_timeout, &errstack, &summary_ad);
 		if (rval != Q_OK) {
-	#else
-		if (fetchUsersFromSchedd (schedd, req_ad, process_ads, process_ads_data, connect_timeout, &errstack, &summary_ad)) {
-	#endif
 			fprintf(stderr, "Error: query failed - %s\n", errstack.getFullText().c_str());
 			exit(1);
 		} else if (dash_long) {
-		#if 0
-			std::string buf; buf.reserve(1024);
-			for (auto it : ads) {
-				classad::References ad_attrs;
-				sGetAdAttrs(ad_attrs, *it.second, false, whitelist);
-				sPrintAdAttrs(buf, *it.second, ad_attrs);
-				buf += "\n";
-				fputs(buf.c_str(), stdout);
-				buf.clear();
-			}
-		#else
 			CondorClassAdListWriter writer(dash_long_format);
 			for (const auto &[key, ad]: ads) { writer.writeAd(*ad, stdout); }
 			if (writer.needsFooter()) { writer.writeFooter(stdout); }
-		#endif
 		} else {
 			std::string line; line.reserve(1024);
 			// render once to set column widths
@@ -611,6 +608,55 @@ main( int argc, const char *argv[] )
 			fprintf(stderr, "Error: %s failed - %s\n", cmd_to_str(cmd), errstack.getFullText().c_str());
 		}
 	#endif
+	} else if (cmd == EDIT_USERREC) {
+		ClassAd * ad = new ClassAd();
+		for (auto & line : edit_args) {
+			if ( ! ad->Insert(line)) {
+				fprintf(stderr, "Error: not a valid classad assigment: %s\n", line);
+				rval = 1;
+				ad->Clear();
+				break;
+			}
+		}
+		if (ad && ad->size() > 0) {
+			ClassAdList adlist;
+			rval = 1; // assume failure
+
+			if (constraint) {
+				if (ad->AssignExpr(ATTR_REQUIREMENTS, constraint)) {
+					adlist.Insert(ad);
+					rval = 0;
+				} else {
+					fprintf(stderr, "Error: invalid constraint : %s\n", constraint);
+				}
+			} else if (usernames.size()) {
+				// we need a separate ad for each user, each should contain all of the edit_args
+				for (auto & name : usernames) {
+					// make a copy of the edit_args attributes for each user beyond the first
+					if (adlist.Length() > 0) { ad = new ClassAd(*ad); }
+					ad->Assign(ATTR_USER, name);
+					adlist.Insert(ad);
+					rval = 0;
+				}
+			} else {
+				fprintf(stderr, "Error: no username for constraint - don't know which user(s) to edit\n");
+				rval = 1;
+			}
+
+			// if we got to here with a valid adlist, send it on to the schedd
+			if (rval == 0) {
+				CondorError errstack;
+				ClassAd * resultAd = schedd.updateUserAds(adlist, &errstack);
+				if ( ! resultAd) {
+					fprintf(stderr, "Error: edit failed - %s\n", errstack.getFullText().c_str());
+					rval = 1;
+				} else {
+					rval = 0;
+					print_results(resultAd, "edit");
+					delete resultAd;
+				}
+			}
+		}
 	} else {
 		fprintf(stderr, "Unsupported command %d\n", cmd);
 		rval = 1;
@@ -793,9 +839,9 @@ usage(FILE *out, const char *appname)
 		exit(1);
 	}
 	fprintf(out, "Usage: %s [ADDRESS] [DISPLAY] [USERS]\n", appname );
-	fprintf(out, "       %s [ADDRESS] [OPERATION] [USERS]\n", appname );
+	fprintf(out, "       %s [ADDRESS] [-add | -enable] [USERS]\n", appname );
 	fprintf(out, "       %s [ADDRESS] -disable [USERS] [-reason <reason-string>]\n", appname );
-	fprintf(out, "       %s [ADDRESS] -edit [USERS] <attr>=<value> [<attr>=<value> ...]\n", appname );
+	fprintf(out, "       %s [ADDRESS] [USERS] -edit <attr>=<value> [<attr>=<value> ...]\n", appname );
 
 	fprintf(out, "\n  ADDRESS is:\n"
 		"    -name <name>\t Name or address of Scheduler\n"
@@ -826,19 +872,24 @@ usage(FILE *out, const char *appname)
 
 	fprintf(out, "\n  USERS is zero or more of:\n"
 		"    <user>\t\t Operate on <user>\n"
+		"    -user <user>\t Operate on <user>\n"
 //		"    -me\t\t\t Operate on the user running the command\n"
-		"    -constraint <expr>\t Operate on users matching the <expr>\n"
+		"    -constraint <expr>\t Operate on users matching the <expr>. Cannot be used with -add.\n"
 		);
 
-	fprintf(out, "\n  OPERATION is one of:\n");
+	fprintf(out, "\n  At most one of the following operation args may be used:\n");
 	fprintf(out, "    -add\t\t Add new, enabled user records\n" );
 	fprintf(out, "    -enable\t\t Enable existing user records, Add new records as needed\n" );
 	fprintf(out, "    -disable\t\t Disable existing user records, user cannot submit jobs\n" );
-	fprintf(out, "    -reason <string>\t Reason for disabling the user. Use with -disable\n" );
 //	fprintf(out, "    -delete\t\t Delete user records\n" );
 //	fprintf(out, "    -reset\t\t Reset user records to default settings and limits\n" );
-//	fprintf(out, "    -edit\t\t Edit fields of user records\n" );
+	fprintf(out, "    -edit\t\t Edit fields of user records\n" );
+
+	fprintf(out, "\n  Other arguments:\n");
+	fprintf(out, "    -reason <string>\t Reason for disabling the user. Use with -disable\n" );
+	fprintf(out, "    <attr>=<expr>\t Store <attr>=<expr> in the user record. Use with -edit\n" );
+
 	fprintf(out, "\n"
-//		"  This tool is use to query, modify and delete User/Owner records in the Schedd.\n"
+		"  This tool is use to query, create and modify User/Owner records in the Schedd.\n"
 		"  The default operation is to query and display users.\n" );
 }
