@@ -256,7 +256,7 @@ WritePerJobHistoryFile(ClassAd* ad, bool useGjid)
 	// first write out the file to the temp_file_name
 	int fd = safe_open_wrapper_follow(temp_file_name.c_str(), O_WRONLY | O_CREAT | O_EXCL, 0644);
 	if (fd == -1) {
-		EXCEPT("error %d (%s) opening per-job history file for job %d.%d\n",
+		EXCEPT("error %d (%s) opening per-job history file for job %d.%d",
 		        errno, strerror(errno), cluster, proc);
 		return;
 	}
@@ -265,7 +265,7 @@ WritePerJobHistoryFile(ClassAd* ad, bool useGjid)
 		int err = errno;
 		close(fd);
 		unlink(temp_file_name.c_str());
-		EXCEPT("error %d (%s) fdopening file stream for per-job history for job %d.%d\n",
+		EXCEPT("error %d (%s) fdopening file stream for per-job history for job %d.%d",
 		        err, strerror(err), cluster, proc);
 		return;
 	}
@@ -280,7 +280,7 @@ WritePerJobHistoryFile(ClassAd* ad, bool useGjid)
 		int err = errno;
 		fclose(fp);
 		unlink(temp_file_name.c_str());
-		EXCEPT("error %d writing per-job history file for job %d.%d\n",
+		EXCEPT("error %d writing per-job history file for job %d.%d",
 		        err, cluster, proc);
 		return;
 	}
@@ -289,7 +289,7 @@ WritePerJobHistoryFile(ClassAd* ad, bool useGjid)
 	// now atomically rename from temp_file_name to file_name
     if (rotate_file(temp_file_name.c_str(), file_name.c_str())) {
 		unlink(temp_file_name.c_str());
-		EXCEPT("error writing per-job history file for job %d.%d (during rename)\n",
+		EXCEPT("error writing per-job history file for job %d.%d (during rename)",
 		        cluster, proc);
     }
 }
