@@ -74,9 +74,4 @@ inline bool add_attrs_from_string_tokens(classad::References & attrs, const std:
 // print attributes to a std::string, returning the result as a const char *
 const char *print_attrs(std::string &out, bool append, const classad::References & attrs, const char * delim);
 
-// copy attrs into stringlist, returns true if list was modified
-// if append is false, list is cleared first.
-// if check_exist is true, items are only added if the are not already in the list. comparison is case-insensitive.
-bool initStringListFromAttrs(StringList & list, bool append, const classad::References & attrs, bool check_exist=false);
-
 
