@@ -23,7 +23,6 @@
 #include "condor_classad.h"
 #include "condor_attributes.h"
 #include "my_hostname.h"
-#include "string_list.h"
 
 #include "classad/classad_distribution.h"
 #include "classad_oldnew.h"
@@ -296,7 +295,7 @@ bool getClassAdEx( Stream *sock, classad::ClassAd& ad, int options)
 					const char * pe = NULL;
 					long long ll = myatoll(rhs, pe);
 					if (*pe == 0 || *pe == '\r' || *pe == '\n') {
-						inserted = ad.InsertLiteral(attr, classad::Literal::MakeLong(ll));
+						inserted = ad.InsertLiteral(attr, classad::Literal::MakeInteger(ll));
 						IF_PROFILE_GETCLASSAD(subtype = 2);
 					}
 				}
@@ -519,7 +518,7 @@ int putClassAd (Stream *sock, const classad::ClassAd& ad, int options, const cla
 			ExprTree * tree = ad.Lookup(*attr);
 			if (tree) {
 				expanded_whitelist.insert(*attr); // the node exists, so add it to the final whitelist
-				if (tree->GetKind() != ExprTree::LITERAL_NODE) {
+				if (dynamic_cast<classad::Literal *>(tree) == nullptr) {
 					ad.GetInternalReferences(tree, expanded_whitelist, false);
 				}
 			}
@@ -597,7 +596,7 @@ int _putClassAd( Stream *sock, const classad::ClassAd& ad, int options,
 
 	classad::ClassAdUnParser	unp;
 	std::string					buf;
-	buf.reserve(8192);
+	buf.reserve(65536);
 	bool send_server_time = false;
 
 	unp.SetOldClassAd( true, true );
@@ -789,6 +788,7 @@ int _putClassAd( Stream *sock, const classad::ClassAd& ad, int options, const cl
 	}
 
 	std::string buf;
+	buf.reserve(65536);
 	bool crypto_is_noop =  sock->prepare_crypto_for_secret_is_noop();
 	for (classad::References::const_iterator attr = whitelist.begin(); attr != whitelist.end(); ++attr) {
 
