@@ -31,7 +31,6 @@
 #include "condor_arglist.h"
 #define CLASSAD_USER_MAP_RETURNS_STRINGLIST 1
 
-#include <sstream>
 #include <unordered_set>
 
 #if defined(UNIX)
@@ -45,7 +44,7 @@ IsStringEnd(const char *str, unsigned off)
 	return (  (str[off] == '\0') || (str[off] == '\n') || (str[off] == '\r')  );
 }
 
-static StringList ClassAdUserLibs;
+static std::vector<std::string> ClassAdUserLibs;
 
 static void registerClassadFunctions();
 static void classad_debug_dprintf(const char *s);
@@ -72,20 +71,17 @@ void ClassAdReconfig()
 
 	char *new_libs = param( "CLASSAD_USER_LIBS" );
 	if ( new_libs ) {
-		StringList new_libs_list( new_libs );
-		free( new_libs );
-		new_libs_list.rewind();
-		char *new_lib;
-		while ( (new_lib = new_libs_list.next()) ) {
-			if ( !ClassAdUserLibs.contains( new_lib ) ) {
-				if ( classad::FunctionCall::RegisterSharedLibraryFunctions( new_lib ) ) {
-					ClassAdUserLibs.append( new_lib );
+		for (const auto& new_lib: StringTokenIterator(new_libs)) {
+			if ( !contains(ClassAdUserLibs, new_lib) ) {
+				if ( classad::FunctionCall::RegisterSharedLibraryFunctions(new_lib.c_str()) ) {
+					ClassAdUserLibs.emplace_back(new_lib);
 				} else {
 					dprintf( D_ALWAYS, "Failed to load ClassAd user library %s: %s\n",
-							 new_lib, classad::CondorErrMsg.c_str() );
+							 new_lib.c_str(), classad::CondorErrMsg.c_str() );
 				}
 			}
 		}
+		free(new_libs);
 	}
 
 	reconfig_user_maps();
@@ -96,12 +92,12 @@ void ClassAdReconfig()
 		std::string user_python(user_python_char);
 		free(user_python_char); user_python_char = NULL;
 		char *loc_char = param("CLASSAD_USER_PYTHON_LIB");
-		if (loc_char && !ClassAdUserLibs.contains(loc_char))
+		if (loc_char && !contains(ClassAdUserLibs, loc_char))
 		{
 			std::string loc(loc_char);
 			if (classad::FunctionCall::RegisterSharedLibraryFunctions(loc.c_str()))
 			{
-				ClassAdUserLibs.append(loc.c_str());
+				ClassAdUserLibs.emplace_back(loc);
 #if defined(UNIX)
 				void *dl_hdl = dlopen(loc.c_str(), RTLD_LAZY);
 				if (dl_hdl) // Not warning on failure as the RegisterSharedLibraryFunctions should have done that.
@@ -218,8 +214,10 @@ bool stringListSize_func( const char * /*name*/,
 		return true;
 	}
 
-	StringList sl( list_str.c_str(), delim_str.c_str() );
-	result.SetIntegerValue( sl.number() );
+	StringTokenIterator sti(list_str, delim_str.c_str());
+	size_t cnt = std::distance(sti.begin(), sti.end());
+
+	result.SetIntegerValue(cnt);
 
 	return true;
 }
@@ -297,8 +295,22 @@ bool stringListSummarize_func( const char *name,
 		return false;
 	}
 
-	StringList sl( list_str.c_str(), delim_str.c_str() );
-	if ( sl.number() == 0 ) {
+	int cnt = 0;
+	for (auto& entry : StringTokenIterator(list_str, delim_str.c_str())) {
+		cnt++;
+		double temp;
+		int r = sscanf(entry.c_str(), "%lf", &temp);
+		if (r != 1) {
+			result.SetErrorValue();
+			return true;
+		}
+		if (strspn(entry.c_str(), "+-0123456789") != entry.size()) {
+			is_real = true;
+		}
+		accumulator = func( temp, accumulator );
+	}
+
+	if ( cnt == 0 ) {
 		if ( empty_allowed ) {
 			result.SetRealValue( 0.0 );
 		} else {
@@ -307,23 +319,8 @@ bool stringListSummarize_func( const char *name,
 		return true;
 	}
 
-	sl.rewind();
-	const char *entry;
-	while ( (entry = sl.next()) ) {
-		double temp;
-		int r = sscanf(entry, "%lf", &temp);
-		if (r != 1) {
-			result.SetErrorValue();
-			return true;
-		}
-		if (strspn(entry, "+-0123456789") != strlen(entry)) {
-			is_real = true;
-		}
-		accumulator = func( temp, accumulator );
-	}
-
 	if ( is_avg ) {
-		accumulator /= sl.number();
+		accumulator /= cnt;
 	}
 
 	if ( is_real ) {
@@ -408,11 +405,11 @@ bool stringListMember_func( const char *name,
 
 	int rc = 0;
 	if (member) {
-		StringList sl( list_str.c_str(), delim_str.c_str() );
+		std::vector<std::string> sl = split(list_str, delim_str.c_str());
 		if (case_sensitive) {
-			rc = sl.contains( item_str.c_str() );
+			rc = contains(sl, item_str.c_str());
 		} else {
-			rc = sl.contains_anycase( item_str.c_str() );
+			rc = contains_anycase(sl, item_str.c_str());
 		}
 	} else if (subset_match) { // SubsetMatch
 
@@ -520,12 +517,6 @@ bool stringListRegexpMember_func( const char * /*name*/,
 		return true;
 	}
 
-	StringList sl( list_str.c_str(), delim_str.c_str() );
-	if ( sl.number() == 0 ) {
-		result.SetUndefinedValue();
-		return true;
-	}
-
 	Regex r;
 	int errcode;
 	int errpos = 0;
@@ -541,14 +532,18 @@ bool stringListRegexpMember_func( const char * /*name*/,
 
 	result.SetBooleanValue( false );
 
-	sl.rewind();
-	char *entry;
-	while( (entry = sl.next())) {
-		if (r.match(entry)) {
+	bool empty = true;
+	for (auto& entry : StringTokenIterator(list_str, delim_str.c_str())) {
+		empty = false;
+		if (r.match(entry.c_str())) {
 			result.SetBooleanValue( true );
 		}
 	}
 
+	if (empty) {
+		result.SetUndefinedValue();
+	}
+
 	return true;
 }
 
@@ -603,7 +598,7 @@ bool userMap_func( const char * /*name*/,
 
 	std::string output;
 	if (user_map_do_mapping(mapName.c_str(), userName.c_str(), output)) {
-		StringList items(output.c_str(), ",");
+		StringTokenIterator items(output, ",");
 
 		if (cargs == 2) {
 			// 2 arg form, return a list.
@@ -623,8 +618,14 @@ bool userMap_func( const char * /*name*/,
 			// preferred item match is case-insensitive.  If the list is empty return undefined
 			std::string pref;
 			const char * selected_item = NULL;
-			const bool any_case = true;
-			if (prefVal.IsStringValue(pref)) { selected_item = items.find(pref.c_str(), any_case); }
+			if (prefVal.IsStringValue(pref)) {
+				for (const char* item = items.first(); item; item = items.next()) {
+					if (strcasecmp(item, pref.c_str()) == 0) {
+						selected_item = item;
+						break;
+					}
+				}
+			}
 			if ( ! selected_item) { selected_item = items.first(); }
 			if (selected_item) {
 				result.SetStringValue(selected_item);
@@ -787,12 +788,9 @@ static void
 problemExpression(const std::string &msg, classad::ExprTree *problem, classad::Value &result)
 {
 	result.SetErrorValue();
-	classad::ClassAdUnParser up;
-	std::string problem_str;
-	up.Unparse(problem_str, problem);
-	std::stringstream ss;
-	ss << msg << "  Problem expression: " << problem_str;
-	classad::CondorErrMsg = ss.str();
+	classad::ClassAdUnParser unp;
+	classad::CondorErrMsg = msg + "  Problem expression: ";
+	unp.Unparse(classad::CondorErrMsg, problem);
 }
 
 
@@ -804,10 +802,8 @@ ArgsToList( const char * name,
 {
 	if ((arguments.size() != 1) && (arguments.size() != 2))
 	{
-		std::stringstream ss;
 		result.SetErrorValue();
-		ss << "Invalid number of arguments passed to " << name << "; one string argument expected.";
-		classad::CondorErrMsg = ss.str();
+		classad::CondorErrMsg = std::string("Invalid number of arguments passed to ") +  name + "; one string argument expected.";
 		return true;
 	}
 	int vers = 2;
@@ -826,9 +822,9 @@ ArgsToList( const char * name,
 		}
 		if ((vers != 1) && (vers != 2))
 		{
-			std::stringstream ss;
-			ss << "Valid values for version are 1 or 2.  Passed expression evaluates to " << vers << ".";
-			problemExpression(ss.str(), arguments[1], result);
+			std::string s;
+			formatstr(s, "Valid values for version are 1 or 2.  Passed expression evaluates to %d.", vers);
+			problemExpression(s, arguments[1], result);
 			return true;
 		}
 	}
@@ -848,16 +844,16 @@ ArgsToList( const char * name,
 	std::string error_msg;
 	if ((vers == 1) && !arg_list.AppendArgsV1Raw(args.c_str(), error_msg))
 	{
-		std::stringstream ss;
-		ss << "Error when parsing argument to arg V1: " << error_msg.c_str();
-		problemExpression(ss.str(), arguments[0], result);
+		std::string s;
+		s = std::string("Error when parsing argument to arg V1: ") + error_msg;
+		problemExpression(s, arguments[0], result);
 		return true;
 	}
 	else if ((vers == 2) && !arg_list.AppendArgsV2Raw(args.c_str(), error_msg))
 	{
-		std::stringstream ss;
-		ss << "Error when parsing argument to arg V2: " << error_msg.c_str();
-		problemExpression(ss.str(), arguments[0], result);
+		std::string s;
+		s = std::string("Error when parsing argument to arg V2: ") + error_msg;
+		problemExpression(s, arguments[0], result);
 		return true;
 	}
 	std::vector<classad::ExprTree*> list_exprs;
@@ -903,10 +899,8 @@ ListToArgs(const char * name,
 {
 	if ((arguments.size() != 1) && (arguments.size() != 2))
 	{
-		std::stringstream ss;
 		result.SetErrorValue();
-		ss << "Invalid number of arguments passed to " << name << "; one list argument expected.";
-		classad::CondorErrMsg = ss.str();
+		classad::CondorErrMsg = std::string("Invalid number of arguments passed to ") + name + "; one list argument expected.";
 		return true;
 	}
 	int vers = 2;
@@ -925,9 +919,9 @@ ListToArgs(const char * name,
 		}
 		if ((vers != 1) && (vers != 2))
 		{
-			std::stringstream ss;
-			ss << "Valid values for version are 1 or 2.  Passed expression evaluates to " << vers << ".";
-			problemExpression(ss.str(), arguments[1], result);
+			std::string s;
+			formatstr(s, "Valid values for version are 1 or 2.  Passed expression evaluates to %d.", vers);
+			problemExpression(s, arguments[1], result);
 			return true;
 		}
 	}
@@ -945,22 +939,22 @@ ListToArgs(const char * name,
 	}
 	ArgList arg_list;
 	size_t idx=0;
-	for (classad::ExprList::const_iterator it=args->begin(); it!=args->end(); it++, idx++)
+	for (auto it=args->begin(); it!=args->end(); it++, idx++)
 	{
 		classad::Value value;
 		if (!(*it)->Evaluate(state, value))
 		{
-			std::stringstream ss;
-			ss << "Unable to evaluate list entry " << idx << ".";
-			problemExpression(ss.str(), *it, result);
+			std::string s;
+			formatstr(s, "Unable to evaluate list entry %zu.", idx);
+			problemExpression(s, *it, result);
 			return false;
 		}
 		std::string tmp_str;
 		if (!value.IsStringValue(tmp_str))
 		{
-			std::stringstream ss;
-			ss << "Entry " << idx << " did not evaluate to a string.";
-			problemExpression(ss.str(), *it, result);
+			std::string s;
+			formatstr(s, "Entry %zu did not evaluate to a string.", idx);
+			problemExpression(s, *it, result);
 			return true;
 		}
 		arg_list.AppendArg(tmp_str.c_str());
@@ -968,16 +962,14 @@ ListToArgs(const char * name,
 	std::string error_msg, result_mystr;
 	if ((vers == 1) && !arg_list.GetArgsStringV1Raw(result_mystr, error_msg))
 	{
-		std::stringstream ss;
-		ss << "Error when parsing argument to arg V1: " << error_msg.c_str();
-		problemExpression(ss.str(), arguments[0], result);
+		std::string s = std::string("Error when parsing argument to arg V1: ") + error_msg;
+		problemExpression(s, arguments[0], result);
 		return true;
 	}
 	else if ((vers == 2) && !arg_list.GetArgsStringV2Raw(result_mystr))
 	{
-		std::stringstream ss;
-		ss << "Error when parsing argument to arg V2: " << error_msg.c_str();
-		problemExpression(ss.str(), arguments[0], result);
+		std::string s = std::string("Error when parsing argument to arg V2: ") +  error_msg;
+		problemExpression(s, arguments[0], result);
 		return true;
 	}
 	result.SetStringValue(result_mystr);
@@ -993,10 +985,9 @@ EnvironmentV1ToV2(const char * name,
 {
 	if (arguments.size() != 1)
 	{
-		std::stringstream ss;
 		result.SetErrorValue();
-		ss << "Invalid number of arguments passed to " << name << "; one string argument expected.";
-		classad::CondorErrMsg = ss.str();
+		classad::CondorErrMsg =
+		   	std::string("Invalid number of arguments passed to ") +  name + "; one string argument expected.";
 		return true;
 	}
 	classad::Value val;
@@ -1044,14 +1035,14 @@ MergeEnvironment(const char * /*name*/,
 {
 	Env env;
 	size_t idx = 0;
-	for (classad::ArgumentList::const_iterator it=arguments.begin(); it!=arguments.end(); it++, idx++)
+	for (auto it=arguments.begin(); it!=arguments.end(); it++, idx++)
 	{
 		classad::Value val;
 		if (!(*it)->Evaluate(state, val))
 		{
-			std::stringstream ss;
-			ss << "Unable to evaluate argument " << idx << ".";
-			problemExpression(ss.str(), *it, result);
+			std::string s;
+			formatstr(s, "Unable to evaluate argument %zu.", idx);
+			problemExpression(s, *it, result);
 			return false;
 		}
 			// Skip over undefined values; this makes it more natural
@@ -1064,16 +1055,16 @@ MergeEnvironment(const char * /*name*/,
 		std::string env_str;
 		if (!val.IsStringValue(env_str))
 		{
-			std::stringstream ss;
-			ss << "Unable to evaluate argument " << idx << ".";
-			problemExpression(ss.str(), *it, result);
+			std::string s;
+			formatstr(s, "Unable to evaluate argument %zu.", idx);
+			problemExpression(s, *it, result);
 			return true;
 		}
 		if (!env.MergeFromV2Raw(env_str.c_str(), nullptr))
 		{
-			std::stringstream ss;
-			ss << "Argument " << idx << " cannot be parsed as environment string.";
-			problemExpression(ss.str(), *it, result);
+			std::string s;
+			formatstr(s, "Argument %zu cannot be parsed as environment string.", idx);
+			problemExpression(s, *it, result);
 			return true;
 		}
 	}
@@ -1133,10 +1124,10 @@ userHome_func(const char *                 name,
 {
 	if ((arguments.size() != 1) && (arguments.size() != 2))
 	{
-		std::stringstream ss;
 		result.SetErrorValue();
-		ss << "Invalid number of arguments passed to " << name << "; " << arguments.size() << "given, 1 required and 1 optional.";
-		classad::CondorErrMsg = ss.str();
+		std::string s;
+		formatstr(s, "Invalid number of arguments passed to %s ; %zu given, 1 required and 1 optional.", name, arguments.size());
+		classad::CondorErrMsg = s;
 		return false;
 	}
 
@@ -1166,11 +1157,10 @@ userHome_func(const char *                 name,
 	}
 	if (!owner_value.IsStringValue(owner_string))
 	{
-		std::string unp_string;
-		std::stringstream ss;
-		classad::ClassAdUnParser unp; unp.Unparse(unp_string, arguments[0]);
-		ss << "Could not evaluate the first argument of " << name << " to string.  Expression: " << unp_string << ".";
-		return return_home_result(default_home, ss.str(), result, true);
+		std::string s = std::string("Could not evaluate the first argument of ") + name + " to string.  Expression: ";
+		classad::ClassAdUnParser unp; unp.Unparse(s, arguments[0]);
+		s += '.';
+		return return_home_result(default_home, s, result, true);
 	}
 
 	errno = 0;
@@ -1186,21 +1176,19 @@ userHome_func(const char *                 name,
 	struct passwd *info = getpwnam(owner_string.c_str());
 	if (!info)
 	{
-		std::stringstream ss;
-		ss << "Unable to find home directory for user " << owner_string;
+		std::string s = std::string("Unable to find home directory for user ") + owner_string;
 		if (errno) {
-			ss << ": " << strerror(errno) << "(errno=" << errno << ")";
+			s += std::string(": ") + strerror(errno) +  "(errno=" + std::to_string(errno) + ')';
 		} else {
-			ss << ": No such user.";
+			s += ": No such user.";
 		}
-		return return_home_result(default_home, ss.str(), result, false);
+		return return_home_result(default_home, s, result, false);
 	}
 
 	if (!info->pw_dir)
 	{
-		std::stringstream ss;
-		ss << "User " << owner_string << " has no home directory.";
-		return return_home_result(default_home, ss.str(), result, false);
+		std::string s = std::string("User ") + owner_string + " has no home directory";
+		return return_home_result(default_home, s, result, false);
 	}
 	std::string home_string = info->pw_dir;
 	result.SetStringValue(home_string);
@@ -2848,12 +2836,6 @@ GetExprReferences( const classad::ExprTree *tree, const classad::ClassAd &ad,
 		return false;
 	}
 
-		// We first process the references and save results in
-		// final_*_refs_set.  The processing may hit duplicates that
-		// are referred to xand then copy from there to the caller's
-		// StringLists.  This scales better than trying to remove
-		// duplicates while inserting into the StringList.
-
 	if ( external_refs ) {
 		TrimReferenceNames( ext_refs_set, true );
 		external_refs->insert( ext_refs_set.begin(), ext_refs_set.end() );
