@@ -22,11 +22,9 @@
 
 #include "classad/classad_distribution.h"
 #include "classad_oldnew.h"
-#include "string_list.h"
 
 using classad::ClassAd;
 
-class StringList;
 class Stream;
 
 #ifndef ATTRLIST_MAX_EXPRESSION
