@@ -39,13 +39,13 @@
 #endif
 
 const int UTIL_MAX_LINE_LENGTH = 1024;
-
 // The default maximum rescue DAG number.
 const int MAX_RESCUE_DAG_DEFAULT = 100;
-
 // The absolute maximum allowed rescue DAG number (the real maximum
 // is normally configured lower).
 const int ABS_MAX_RESCUE_DAG_NUM = 999;
+// MaxIdle Jobs default value
+const int MAX_IDLE_DEFAULT = 1000;
 
 enum DagStatus {
     DAG_STATUS_OK = 0,
@@ -57,6 +57,11 @@ enum DagStatus {
     DAG_STATUS_HALTED = 6, // DAG was halted and submitted jobs finished
 };
 
+enum class DagSubmitMethod {
+	CONDOR_SUBMIT = 0,
+	DIRECT = 1
+};
+
 #include "enum.h"
 typedef std::list<std::string> str_list;
 
@@ -134,16 +139,17 @@ struct CLI_BOOL_FLAG {
 namespace DagmanShallowOptions {
 	BETTER_ENUM(str, long,
 		ScheddDaemonAdFile = 0, ScheddAddressFile, ConfigFile, SaveFile, RemoteSchedd, AppendFile,
-		PrimaryDagFile, LibOut, LibErr, DebugLog, SchedLog, SubFile, RescueFile, LockFile
+		PrimaryDagFile, LibOut, LibErr, DebugLog, SchedLog, SubFile, RescueFile, LockFile,
+		CsdVersion
 	);
 
 	BETTER_ENUM(i, long,
-		MaxIdle = 0, MaxJobs, MaxPre, MaxPost, DebugLevel, Priority
+		MaxIdle = 0, MaxJobs, MaxPre, MaxPost, MaxHold, DebugLevel, Priority
 	);
 
 	BETTER_ENUM(b, long,
 		PostRun = 0, DumpRescueDag, RunValgrind, DoSubmit, DoRecovery, CopyToSpool, DryRun,
-		WaitForDebug
+		OnlyDumpDot
 	);
 
 	BETTER_ENUM(slist, long,
@@ -158,7 +164,7 @@ namespace DagmanDeepOptions {
 	);
 
 	BETTER_ENUM(i, long,
-		DoRescueFrom = 0, AutoRescue
+		DoRescueFrom = 0, AutoRescue, SubmitMethod
 	);
 
 	BETTER_ENUM(b, long,
@@ -178,10 +184,10 @@ struct KeyNoCaseCmp {
 	}
 };
 
-#define DAG_OPT_DISP_DAGMAN (1 << 0)     // Display option for condor_dagman
-#define DAG_OPT_DISP_CSD (1 << 1)        // Display option for condor_submit_dag
-#define DAG_OPT_DISP_PY_BIND (1 << 2)    // Display option for python bindings
-#define DAG_OPT_DISP_ALL INT_MAX         // Display option for all sources
+const int DAG_OPT_DISP_DAGMAN = (1 << 0);     // Display option for condor_dagman
+const int DAG_OPT_DISP_CSD = (1 << 1);        // Display option for condor_submit_dag
+const int DAG_OPT_DISP_PY_BIND = (1 << 2);    // Display option for python bindings
+const int DAG_OPT_DISP_ALL = INT_MAX;         // Display option for all sources
 
 typedef std::tuple<std::string, std::string, std::string, int> DagOptionInfo;
 static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMap = {
@@ -192,6 +198,7 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	{"-AutoRescue",    {"AutoRescue", "<0|1>", "Control automatically running new rescue DAG (0=False | 1=True)", DAG_OPT_DISP_ALL}}, // Note non-zero = True
 	{"-Batch-Name",    {"BatchName", "<name>", "Set DAG batch name", DAG_OPT_DISP_ALL}},
 	{"-Config",        {"ConfigFile", "<filename>", "Specify DAGMan configuration file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
+	{"-CsdVersion",    {"CsdVersion", "<Version String>", "Current condor_submit_dag version string", 0}}, // Not user controlled and manually displayed in condor_dagman
 	{"-Dag",           {"DagFiles", "<NAME.dag>", "DAG file for DAGMan to execute", 0}}, // Only applies to DAGMan Main and is manually entered
 	{"-DAGMan",        {"DagmanPath", "<path>", "Full path to alternate condor_dagman executable", DAG_OPT_DISP_ALL}},
 	{"-Debug",         {"DebugLevel", "<level>", "Set DAGMan debug logs verbosity", DAG_OPT_DISP_ALL}},
@@ -200,9 +207,11 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	{"-DontAlwaysRunPost", {"PostRun", "False", "Don't run POST script if PRE script fails", DAG_OPT_DISP_ALL}},
 	{"-DoRecovery",    {"DoRecovery", "True", "Allow DAG submitted jobs to send email notifications", DAG_OPT_DISP_ALL}},
 	{"-DoRescueFrom",  {"DoRescueFrom", "<N>", "Run DAG rescue of given number", DAG_OPT_DISP_ALL}},
+	{"-Dot",           {"OnlyDumpDot", "True", "Have DAGMan dump DOT file and exit", DAG_OPT_DISP_DAGMAN}},
 	{"-DryRun",        {"DryRun", "True", "Dry run condor_dagman execution of DAG", DAG_OPT_DISP_DAGMAN}},
 	{"-DumpRescue",    {"DumpRescueDag", "True", "DAGMan dump rescue DAG and exit", DAG_OPT_DISP_ALL}},
-	{"-Force",         {"Force", "True", "Overwrite used DAG file if they exist", DAG_OPT_DISP_ALL}}, // Note: -f works for this
+	{"-f",             {"Force", "True", "See -Force", 0}}, // Single letter flag to make -f equal to -Force
+	{"-Force",         {"Force", "True", "Overwrite used DAG file if they exist", DAG_OPT_DISP_ALL}},
 	{"-import_env",    {"ImportEnv", "True", "Import current environment into *.condor.sub file", DAG_OPT_DISP_ALL}},
 	{"-include_env",   {"GetFromEnv", "<variables>", "Comma separated list of environment variables to *.condor.sub file getenv filter", DAG_OPT_DISP_ALL}},
 	{"-insert_env",    {"AddToEnv", "<key=value>", "Delimited key=value pairs to explicitly set in the *.condor.sub file environment", DAG_OPT_DISP_ALL}},
@@ -210,6 +219,7 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	{"-load_save",     {"SaveFile", "<filename>", "Run DAG from provided save point file", DAG_OPT_DISP_ALL}},
 	{"-Lockfile",      {"LockFile", "<NAME.dag.lock>", "DAGMan lock filename", 0}}, // Only applies to DAGMan Main and is manually displayed
 	{"-MaxIdle",       {"MaxIdle", "<N>", "Maximum number of Idle nodes allowed", DAG_OPT_DISP_ALL}},
+	{"-MaxHold",       {"MaxHold", "<N>", "Maximum number of HOLD scripts to run at once", DAG_OPT_DISP_ALL}},
 	{"-MaxJobs",       {"MaxJobs", "<N>", "Maximum number of jobs submitted at once", DAG_OPT_DISP_ALL}},
 	{"-MaxPost",       {"MaxPost", "<N>", "Maximum number of POST scripts to run at once", DAG_OPT_DISP_ALL}},
 	{"-MaxPre",        {"MaxPre", "<N>", "Maximum number of PRE scripts to run at once", DAG_OPT_DISP_ALL}},
@@ -222,12 +232,13 @@ static const std::map<std::string, DagOptionInfo, KeyNoCaseCmp> dagOptionsInfoMa
 	{"-schedd-address-file", {"ScheddAddressFile", "<path>", "Submit DAG to Schedd provided by address file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-schedd-daemon-ad-file", {"ScheddDaemonAdFile", "<path>", "Submit DAG to Schedd provided by ad file", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-suppress_notification", {"SuppressNotification", "True", "Suppress email notifications for DAGMan and all its submitted jobs", DAG_OPT_DISP_ALL}},
+	{"-SubmitMethod",  {"SubmitMethod", "<value>", "Specify how DAGMan submits jobs for execution (0=condor_submit|1=DirectSubmit)", DAG_OPT_DISP_ALL}},
 	{"-update_submit", {"UpdateSubmit", "True", "Update *.condor.sub file if it exists", DAG_OPT_DISP_ALL}},
 	{"-UseDagDir",     {"UseDagDir", "True", "Run DAGs in directories specified by DAG file paths", DAG_OPT_DISP_ALL}},
 	{"-v",             {"Verbose", "True", "See -Verbose", 0}}, // Single letter flag to make -v equal to -Verbose
 	{"-Valgrind",      {"RunValgrind", "True", "Run DAGMan under Valgrind (Linux Only)", DAG_OPT_DISP_CSD|DAG_OPT_DISP_PY_BIND}},
 	{"-Verbose",       {"Verbose", "True", "Increase error message verbosity for condor_submit_dag", DAG_OPT_DISP_CSD}}, // Note: -v works for this
-	{"-WaitForDebug",  {"WaitForDebug", "True", "Pause condor_dagman execution until debugger is attached", DAG_OPT_DISP_DAGMAN}},
+	//{"-WaitForDebug",  {"WaitForDebug", "True", "Pause condor_dagman execution until debugger is attached", DAG_OPT_DISP_DAGMAN}}, // Note: Handled in condor_dagman
 };
 
 enum class DagOptionSrc {
@@ -276,7 +287,7 @@ public:
 			shallow.stringOpts[str::AppendFile] = appendFile;
 			shallow.boolOpts[b::DoSubmit] = true;
 			shallow.boolOpts[b::CopyToSpool] = param_boolean( "DAGMAN_COPY_TO_SPOOL", false );
-			shallow.intOpts[i::MaxIdle] = 0;
+			shallow.intOpts[i::MaxIdle] = MAX_IDLE_DEFAULT;
 			shallow.intOpts[i::MaxJobs] = 0;
 			shallow.intOpts[i::MaxPre] = 0;
 			shallow.intOpts[i::MaxPost] = 0;
@@ -287,7 +298,8 @@ public:
 		{ //Initialize Deep Options
 			using namespace DagmanDeepOptions;
 			deep.intOpts[i::DoRescueFrom] = 0;
-			deep.intOpts[i::AutoRescue] = (int)param_boolean( "DAGMAN_AUTO_RESCUE", true );
+			deep.intOpts[i::AutoRescue] = (int)param_boolean("DAGMAN_AUTO_RESCUE", true);
+			deep.intOpts[i::SubmitMethod] = -1;
 		} //End Deep Option Initialization
 	}
 
@@ -304,7 +316,7 @@ public:
 	// Handle special case modifications/messages regarding specific options
 	std::string processOptionArg(const std::string& opt, std::string arg);
 	// Automagically parse cli flags to set a DAGMan option (Note: Possible incrementation of iArg)
-	bool AutoParse(const std::string &flag, size_t &iArg, const size_t argc, const char * const argv[], std::string &err);
+	bool AutoParse(const std::string &flag, size_t &iArg, const size_t argc, const char * const argv[], std::string &err, DagmanOptions* duplicate=nullptr);
 
 	// Extend (push_back) value to list option
 	// Added to help clear up possible confusion from set() despite set() doing the correct thing
@@ -317,7 +329,7 @@ public:
 	inline std::string primaryDag() const { return shallow.stringOpts[DagmanShallowOptions::str::PrimaryDagFile]; }
 	inline str_list dagFiles() const { return shallow.slistOpts[DagmanShallowOptions::slist::DagFiles]; }
 	inline bool isMultiDag() const { return is_MultiDag; }
-
+	size_t numDagFiles() const { return shallow.slistOpts[DagmanShallowOptions::slist::DagFiles].size(); }
 
 	// Const shallow options access operator declarations
 	const str_list & operator[]( SSO::slist opt ) const { return shallow.slistOpts[opt._to_integral()]; }
@@ -377,6 +389,8 @@ public:
 
 	bool ensureOutputFilesExist(const DagmanOptions &options);
 
+	std::tuple<std::string, bool> ResolveSaveFile(const std::string& primaryDag, const std::string& saveFile, bool mkSaveDir=false);
+
 	std::string RescueDagName(const std::string &primaryDagFile, bool multiDags, int rescueDagNum);
 
 	void RenameRescueDagsAfter(const std::string &primaryDagFile, bool multiDags, int rescueDagNum, int maxRescueDagNum);
