@@ -288,7 +288,12 @@ DataReuseDirectory::UpdateState(LogSentry &sentry, CondorError &err)
 	bool all_done = false;
 	do {
 		ULogEvent *event = nullptr;
+	#if 0  // can no longer pass a lock into the userlog reader
 		auto outcome = m_rlog.readEventWithLock(event, *sentry.lock());
+	#else
+		auto outcome = m_rlog.readEvent(event);
+	#endif
+
 
 		switch (outcome) {
 		case ULOG_OK:
