@@ -381,7 +381,10 @@ bool userlog_to_classads(const char *filename,
 	    jobClassAd->InsertAttr("LastVacateTime",eventTime);
 	}
 
-	jobClassAd->InsertAttr("LastVacateReason",evict_event->message);
+	{
+	  const char * reason = evict_event->getMessage();
+	  if (reason) { jobClassAd->InsertAttr("LastVacateReason", reason); }
+	}
 
 	// ignore file trasfer for now
       }
@@ -412,8 +415,9 @@ bool userlog_to_classads(const char *filename,
 	}
 
 	{
-	  if (!evict_event->reason.empty()) {
-	    jobClassAd->InsertAttr("LastVacateReason",evict_event->reason);
+	  const char * reason = evict_event->getReason();
+	  if (reason && reason[0]) {
+	    jobClassAd->InsertAttr("LastVacateReason",reason);
 	  }
 	}
       }
@@ -439,15 +443,15 @@ bool userlog_to_classads(const char *filename,
 
 	// Update the time attributes
 	{
-	  const char * reason=hold_event->reason.c_str();
+	  const char * reason=hold_event->getReason();
 
 	  {
 	    condor_time_t eventTime = getEventTime(event);
 	    jobClassAd->InsertAttr("EnteredCurrentStatus",eventTime);
 	    if (prev_status!=IDLE) {
 	      jobClassAd->InsertAttr("LastVacateTime",eventTime);
-	      if (reason!=NULL) {
-		jobClassAd->InsertAttr("LastVacateReason",reason);
+	      if (reason && reason[0]) {
+	         jobClassAd->InsertAttr("LastVacateReason",reason);
 	      }
 	    }
 	  }
@@ -490,8 +494,9 @@ bool userlog_to_classads(const char *filename,
 	}
 
 	{
-	  if (!rel_event->reason.empty()) {
-	    jobClassAd->InsertAttr("ReleaseReason",rel_event->reason);
+	  const char * reason = rel_event->getReason();
+	  if (reason && reason[0]) {
+	    jobClassAd->InsertAttr("ReleaseReason",reason);
 	  }
 	}
       }
@@ -516,8 +521,9 @@ bool userlog_to_classads(const char *filename,
 	}
 
 	{
-	  if (!abort_event->reason.empty()) {
-	    jobClassAd->InsertAttr("RemoveReason",abort_event->reason);
+	  const char * reason = abort_event->getReason();
+	  if (reason && reason[0]) {
+	    jobClassAd->InsertAttr("RemoveReason",reason);
 	  }
 	}
       }
