@@ -38,7 +38,6 @@
 
 #include <string>
 #include <algorithm>
-#include "condor_attributes.h"
 #include "basename.h"
 
 // Set to non-zero to enable fine-grained rotation debugging / timing
@@ -964,8 +963,12 @@ WriteUserLog::checkGlobalLogRotation( void )
 				 m_global_path, errno, strerror(errno) );
 	}
 	else {
-		bool is_xml = (m_global_format_opts & ULogEvent::formatOpt::XML) != 0;
-		ReadUserLog	log_reader( fp, is_xml, false );
+		UserLogType is_xml = UserLogType::LOG_TYPE_NORMAL;
+		// TODO: add support for JSON and CLASSAD formats
+		if (m_global_format_opts & ULogEvent::formatOpt::XML) {
+			is_xml = UserLogType::LOG_TYPE_XML;
+		}
+		ReadUserLog log_reader( fp, is_xml, false );
 		if ( header_reader.Read( log_reader ) != ULOG_OK ) {
 			dprintf( D_ALWAYS,
 					 "WriteUserLog: Error reading header of \"%s\"\n",
