@@ -28,7 +28,6 @@
 #include "condor_adtypes.h"
 #include "daemon.h"
 
-#include "string_list.h"
 #include "my_username.h" // for my_domainname
 #include "directory.h"
 #include "filename_tools.h"
@@ -39,9 +38,6 @@
 #include "condor_regex.h"
 #include "xform_utils.h"
 
-#include "list.h"
-#include "my_popen.h"
-
 #include <charconv>
 #include <string>
 #include <set>
@@ -776,61 +772,70 @@ int MacroStreamXFormSource::setUniverse(const char * uni) {
 	return universe;
 }
 
-int MacroStreamXFormSource::open(StringList & lines, const MACRO_SOURCE & FileSource, std::string & errmsg)
+int MacroStreamXFormSource::open(std::vector<std::string> &lines, const MACRO_SOURCE & FileSource, std::string & errmsg)
 {
 	std::string   hereTag;
 
 	// process and remove meta statements.  NAME, REQUIREMENTS, UNIVERSE and TRANSFORM
-	for (const char *line = lines.first(); line; line = lines.next()) {
-		const char * p;
+	auto line_it = lines.begin();
+	while (line_it != lines.end()) {
+		std::string line = *line_it;
+		const char * p = nullptr;
 
 		// if we are processing a here @= variable, we don't
 		// want to remove at the lines inside the variable
 		if ( ! hereTag.empty()) {
-			p = line;
+			p = line.c_str();
 			while (*p && isspace(*p)) ++p;
 			if (hereTag == p) {
 				hereTag.clear();
 			}
+			line_it++;
 			continue;
-		} else if (NULL != (p = is_herefile_statement(line))) {
+		} if (nullptr != (p = is_herefile_statement(line.c_str()))) {
 			hereTag = '@'; hereTag += p;
 			trim(hereTag);
+			line_it++;
 			continue;
 		}
 
-		if (NULL != (p = is_xform_statement(line, "name"))) {
+		if (nullptr != (p = is_xform_statement(line.c_str(), "name"))) {
 			std::string tmp(p); trim(tmp);
 			if ( ! tmp.empty()) name = tmp;
-			lines.deleteCurrent();
-		} else if (NULL != (p = is_xform_statement(line, "requirements"))) {
+			line_it = lines.erase(line_it);
+			line_it--;
+		} else if (nullptr != (p = is_xform_statement(line.c_str(), "requirements"))) {
 			int err = 0;
 			setRequirements(p, err);
 			if (err < 0) {
 				formatstr(errmsg, "invalid REQUIREMENTS : %s", p);
 				return err;
 			}
-			lines.deleteCurrent();
-		} else if (NULL != (p = is_xform_statement(line, "universe"))) {
+			line_it = lines.erase(line_it);
+			line_it--;
+		} else if (NULL != (p = is_xform_statement(line.c_str(), "universe"))) {
 			setUniverse(p);
-			lines.deleteCurrent();
-		} else if (NULL != (p = is_xform_statement(line, "transform"))) {
+			line_it = lines.erase(line_it);
+			line_it--;
+		} else if (NULL != (p = is_xform_statement(line.c_str(), "transform"))) {
 			if ( ! iterate_args) {
 				p = is_non_trivial_iterate(p);
 				if (p) { iterate_args.set(strdup(p)); iterate_init_state = 2; }
 			}
-			lines.deleteCurrent();
+			line_it = lines.erase(line_it);
+			line_it--;
 		} else {
 			// strip blank lines and comments
 			//while (*p && isspace(*line)) ++p;
 			//if ( ! *p || *p == '#') { lines.deleteCurrent(); }
 		}
+		line_it++;
 	}
 
-	file_string.set(lines.print_to_delimed_string("\n"));
+	file_string.set(strdup(join(lines,"\n").c_str()));
 	MacroStreamCharSource::open(file_string, FileSource);
 	rewind();
-	return lines.number();
+	return lines.size();
 }
 
 // like the above open, but more efficient when loading from a config value
@@ -841,7 +846,7 @@ int MacroStreamXFormSource::open(const char * statements_in, int & offset, std::
 	size_t cb = strlen(statements);
 	char * buf = (char*)malloc(cb + 2);
 	file_string.set(buf);
-	StringTokenIterator lines(statements, "\r\n", false);
+	StringTokenIterator lines(statements, "\r\n", STI_NO_TRIM);
 	int start, length, linecount = 0;
 	while ((start = lines.next_token(length)) >= 0) {
 		// tentatively copy the line, then append a null terminator
@@ -927,7 +932,7 @@ int MacroStreamXFormSource::open(const char * statements_in, int & offset, std::
 //
 int MacroStreamXFormSource::load(FILE* fp, MACRO_SOURCE & FileSource, std::string & errmsg)
 {
-	StringList lines;
+	std::vector<std::string> lines;
 
 	while (true) {
 		int lineno = FileSource.line;
@@ -949,9 +954,9 @@ int MacroStreamXFormSource::load(FILE* fp, MACRO_SOURCE & FileSource, std::strin
 			// if we read more than a single line, comment the new linenumber
 			std::string buf = "#opt:lineno:";
 			buf += std::to_string(FileSource.line);
-			lines.append(buf.c_str());
+			lines.emplace_back(buf);
 		}
-		lines.append(line);
+		lines.emplace_back(line);
 
 		const char * is_transform = is_xform_statement(line, "transform");
 		if (is_transform) {
@@ -996,7 +1001,7 @@ const char * MacroStreamXFormSource::getFormattedText(std::string & buf, const c
 		buf += requirements.c_str();
 	}
 	if (file_string) {
-		StringTokenIterator lines(file_string.ptr(), "\n", false);
+		StringTokenIterator lines(file_string.ptr(), "\n", STI_NO_TRIM);
 		for (const char * line = lines.first(); line; line = lines.next()) {
 			if ( ! include_comments) {
 				while (*line && isspace(*line)) ++line;
@@ -1048,7 +1053,7 @@ int MacroStreamXFormSource::parse_iterate_args(char * pargs, int expand_options,
 	}
 
 	// if no loop variable specified, but a foreach mode is used. use "Item" for the loop variable.
-	if (oa.vars.isEmpty() && (oa.foreach_mode != foreach_not)) { oa.vars.append("Item"); }
+	if (oa.vars.empty() && (oa.foreach_mode != foreach_not)) { oa.vars.emplace_back("Item"); }
 
 	// fill in the items array from a file
 	if ( ! oa.items_filename.empty()) {
@@ -1067,9 +1072,11 @@ int MacroStreamXFormSource::parse_iterate_args(char * pargs, int expand_options,
 				if (line[0] == '#') continue; // skip comments.
 				if (line[0] == ')') { saw_close_brace = true; break; }
 				if (oa.foreach_mode == foreach_from) {
-					oa.items.append(line);
+					oa.items.emplace_back(line);
 				} else {
-					oa.items.initializeFromString(line);
+					for (const auto& item: StringTokenIterator(line)) {
+						oa.items.emplace_back(item);
+					}
 				}
 			}
 			if (close_fp_when_done) { fclose(fp); fp = NULL; }
@@ -1084,9 +1091,11 @@ int MacroStreamXFormSource::parse_iterate_args(char * pargs, int expand_options,
 				line = getline_trim(stdin, lineno);
 				if ( ! line) break;
 				if (oa.foreach_mode == foreach_from) {
-					oa.items.append(line);
+					oa.items.emplace_back(line);
 				} else {
-					oa.items.initializeFromString(line);
+					for (const auto& item: StringTokenIterator(line)) {
+						oa.items.emplace_back(item);
+					}
 				}
 			}
 		} else {
@@ -1098,7 +1107,7 @@ int MacroStreamXFormSource::parse_iterate_args(char * pargs, int expand_options,
 			for (char* line=NULL;;) {
 				line = getline_trim(fpItems, ItemsSource.line);
 				if ( ! line) break;
-				oa.items.append(line);
+				oa.items.emplace_back(line);
 			}
 			rval = Close_macro_source(fpItems, ItemsSource, set.macros(), 0);
 		}
@@ -1113,7 +1122,7 @@ int MacroStreamXFormSource::parse_iterate_args(char * pargs, int expand_options,
 	case foreach_from:
 		// itemlist is already correct
 		// PRAGMA_REMIND("do argument validation here?")
-		citems = oa.items.number();
+		citems = (int)oa.items.size();
 		break;
 
 	case foreach_matching:
@@ -1181,7 +1190,7 @@ static char EmptyItemString[] = "";
 
 int MacroStreamXFormSource::set_iter_item(XFormHash &set, const char* item)
 {
-	if (oa.vars.isEmpty()) return 0;
+	if (oa.vars.empty()) return 0;
 
 	// make a copy of the item so we can destructively edit it.
 	char * data;
@@ -1196,8 +1205,8 @@ int MacroStreamXFormSource::set_iter_item(XFormHash &set, const char* item)
 
 	// set the first loop variable unconditionally, we set it initially to the whole item
 	// we may later truncate that item when we assign fields to other loop variables.
-	char * var = oa.vars.first();
-	set.set_live_variable(var, data, ctx);
+	auto var_it = oa.vars.begin();
+	set.set_live_variable(var_it->c_str(), data, ctx);
 
 	const char* token_seps = ", \t";
 	const char* token_ws = " \t";
@@ -1205,7 +1214,7 @@ int MacroStreamXFormSource::set_iter_item(XFormHash &set, const char* item)
 	// if there is more than a single loop variable, then assign them as well
 	// we do this by destructively null terminating the item for each var
 	// the last var gets all of the remaining item text (if any)
-	while ((var = oa.vars.next())) {
+	while ((++var_it != oa.vars.end())) {
 		// scan for next token separator
 		while (*data && ! strchr(token_seps, *data)) ++data;
 		// null terminate the previous token and advance to the start of the next token.
@@ -1213,7 +1222,7 @@ int MacroStreamXFormSource::set_iter_item(XFormHash &set, const char* item)
 			*data++ = 0;
 			// skip leading separators and whitespace
 			while (*data && strchr(token_ws, *data)) ++data;
-			set.set_live_variable(var, data, ctx);
+			set.set_live_variable(var_it->c_str(), data, ctx);
 		}
 	}
 	return curr_item.ptr() != NULL;
@@ -1238,8 +1247,12 @@ bool  MacroStreamXFormSource::first_iteration(XFormHash &set)
 	checkpoint = set.save_state();
 
 	// prime the iteration variables
-	oa.items.rewind();
-	return set_iter_item(set, oa.items.next()) || (oa.queue_num > 1);
+	oa.items_idx = 0;
+	const char* item_str = nullptr;
+	if (oa.items_idx < oa.items.size()) {
+		item_str = oa.items[oa.items_idx++].c_str();
+	}
+	return set_iter_item(set, item_str) || (oa.queue_num > 1);
 }
 
 bool MacroStreamXFormSource::next_iteration(XFormHash &set)
@@ -1254,7 +1267,11 @@ bool MacroStreamXFormSource::next_iteration(XFormHash &set)
 		step = 0;
 		++row;
 		if (checkpoint) { set.rewind_to_state(checkpoint, false); }
-		has_next_item = set_iter_item(set, oa.items.next());
+		const char* item_str = nullptr;
+		if (oa.items_idx < oa.items.size()) {
+			item_str = oa.items[oa.items_idx++].c_str();
+		}
+		has_next_item = set_iter_item(set, item_str);
 		set.set_iterate_row(row, true);
 	}
 	set.set_iterate_step(step, proc);
@@ -1269,7 +1286,7 @@ void MacroStreamXFormSource::clear_iteration(XFormHash &set)
 	}
 	set.clear_live_variables();
 	curr_item.clear();
-	oa.items.rewind();
+	oa.items_idx = 0;
 }
 
 void MacroStreamXFormSource::reset(XFormHash &set)
@@ -1985,7 +2002,7 @@ typedef std::map<std::string, std::string, classad::CaseIgnLTStr> STRING_MAP;
 #define XForm_ConvertJobRouter_Old_CE          0x00004
 
 int ConvertClassadJobRouterRouteToXForm (
-	StringList & statements,
+	std::vector<std::string> & statements,
 	std::string & name, // name from config on input, overwritten with name from route ad if it has one
 	const std::string & routing_string,
 	int & offset,
@@ -2185,26 +2202,26 @@ int ConvertClassadJobRouterRouteToXForm (
 
 	std::string buf;
 	formatstr(buf, "# autoconversion of route '%s' from old route syntax", name.c_str());
-	statements.append(buf.c_str());
+	statements.emplace_back(buf.c_str());
 	if ( ! name.empty()) {
 		if ((options & XForm_ConvertJobRouter_Old_CE) && IsValidAttrName(name.c_str())) {
 			// no need to add this to the route text
 			// formatstr(buf, "# NAME %s", name.c_str());
 		} else {
 			formatstr(buf, "NAME %s", name.c_str());
-			statements.append(buf.c_str());
+			statements.emplace_back(buf.c_str());
 		}
 	}
-	if (target_universe) { formatstr(buf, "UNIVERSE %d", target_universe); statements.append(buf.c_str()); }
+	if (target_universe) { formatstr(buf, "UNIVERSE %d", target_universe); statements.emplace_back(buf.c_str()); }
 	if (!requirements.empty()) {
 		formatstr(buf, "REQUIREMENTS %s", requirements.c_str());
-		statements.append(buf.c_str()); 
+		statements.emplace_back(buf.c_str()); 
 	}
 
-	statements.append("");
+	statements.emplace_back();
 	for (STRING_MAP::iterator it = assignments.begin(); it != assignments.end(); ++it) {
 		formatstr(buf, "%s = %s", it->first.c_str(), it->second.c_str());
-		statements.append(buf.c_str());
+		statements.emplace_back(buf);
 	}
 
 // evaluation order of route rules:
@@ -2213,52 +2230,52 @@ int ConvertClassadJobRouterRouteToXForm (
 //3. set_* 
 //4. eval_set_* 
 	if ( ! copy_cmds.empty()) {
-		statements.append("");
-		statements.append("# copy_* rules");
+		statements.emplace_back();
+		statements.emplace_back("# copy_* rules");
 		for (STRING_MAP::iterator it = copy_cmds.begin(); it != copy_cmds.end(); ++it) {
 			formatstr(buf, "COPY %s %s", it->first.c_str(), it->second.c_str());
-			statements.append(buf.c_str());
+			statements.emplace_back(buf);
 		}
 	}
 
 	if ( ! delete_cmds.empty()) {
-		statements.append("");
-		statements.append("# delete_* rules");
+		statements.emplace_back();
+		statements.emplace_back("# delete_* rules");
 		for (STRING_MAP::iterator it = delete_cmds.begin(); it != delete_cmds.end(); ++it) {
 			formatstr(buf, "DELETE %s", it->first.c_str());
-			statements.append(buf.c_str());
+			statements.emplace_back(buf.c_str());
 		}
 	}
 
 	if ( ! set_cmds.empty()) {
-		statements.append("");
-		statements.append("# set_* rules");
+		statements.emplace_back();
+		statements.emplace_back("# set_* rules");
 		for (STRING_MAP::iterator it = set_cmds.begin(); it != set_cmds.end(); ++it) {
 			formatstr(buf, "SET %s %s", it->first.c_str(), it->second.c_str());
-			statements.append(buf.c_str());
+			statements.emplace_back(buf);
 		}
 	}
 
 	if (has_def_onexithold && (options & XForm_ConvertJobRouter_Fix_EvalSet)) {
 		// emit new boilerplate on_exit_hold
 		if (has_set_minWallTime) {
-			statements.append("");
-			statements.append("# modify OnExitHold for minWallTime");
-			statements.append("if defined MY.OnExitHold");
-			statements.append("  COPY OnExitHold orig_OnExitHold");
-			statements.append("  COPY OnExitHoldSubCode orig_OnExitHoldSubCode");
-			statements.append("  COPY OnExitHoldReason orig_OnExitHoldReason");
-			statements.append("  DEFAULT orig_OnExitHoldReason strcat(\"The on_exit_hold expression (\", unparse(orig_OnExitHold), \") evaluated to TRUE.\")");
-			statements.append("  SET OnExitHoldMinWallTime ifThenElse(RemoteWallClockTime isnt undefined, RemoteWallClockTime < 60*$(minWallTime), false)");
-			statements.append("  SET OnExitHoldReasonMinWallTime strcat(\"The job's wall clock time\", int(RemoteWallClockTime/60), \"min, is is less than the minimum specified by the job ($(minWallTime))\")");
-			statements.append("  SET OnExitHold orig_OnExitHold || OnExitHoldMinWallTime");
-			statements.append("  SET OnExitHoldSubCode ifThenElse(orig_OnExitHold, $(My.orig_OnExitHoldSubCode:1), 42)");
-			statements.append("  SET OnExitHoldReason ifThenElse(orig_OnExitHold, orig_OnExitHoldReason, ifThenElse(OnExitHoldMinWallTime, OnExitHoldReasonMinWallTime, \"Job held for unknown reason.\"))");
-			statements.append("else");
-			statements.append("  SET OnExitHold ifThenElse(RemoteWallClockTime isnt undefined, RemoteWallClockTime < 60*$(minWallTime), false)");
-			statements.append("  SET OnExitHoldSubCode 42");
-			statements.append("  SET OnExitHoldReason strcat(\"The job's wall clock time\", int(RemoteWallClockTime/60), \"min, is is less than the minimum specified by the job ($(minWallTime))\")");
-			statements.append("endif");
+			statements.emplace_back("");
+			statements.emplace_back("# modify OnExitHold for minWallTime");
+			statements.emplace_back("if defined MY.OnExitHold");
+			statements.emplace_back("  COPY OnExitHold orig_OnExitHold");
+			statements.emplace_back("  COPY OnExitHoldSubCode orig_OnExitHoldSubCode");
+			statements.emplace_back("  COPY OnExitHoldReason orig_OnExitHoldReason");
+			statements.emplace_back("  DEFAULT orig_OnExitHoldReason strcat(\"The on_exit_hold expression (\", unparse(orig_OnExitHold), \") evaluated to TRUE.\")");
+			statements.emplace_back("  SET OnExitHoldMinWallTime ifThenElse(RemoteWallClockTime isnt undefined, RemoteWallClockTime < 60*$(minWallTime), false)");
+			statements.emplace_back("  SET OnExitHoldReasonMinWallTime strcat(\"The job's wall clock time\", int(RemoteWallClockTime/60), \"min, is is less than the minimum specified by the job ($(minWallTime))\")");
+			statements.emplace_back("  SET OnExitHold orig_OnExitHold || OnExitHoldMinWallTime");
+			statements.emplace_back("  SET OnExitHoldSubCode ifThenElse(orig_OnExitHold, $(My.orig_OnExitHoldSubCode:1), 42)");
+			statements.emplace_back("  SET OnExitHoldReason ifThenElse(orig_OnExitHold, orig_OnExitHoldReason, ifThenElse(OnExitHoldMinWallTime, OnExitHoldReasonMinWallTime, \"Job held for unknown reason.\"))");
+			statements.emplace_back("else");
+			statements.emplace_back("  SET OnExitHold ifThenElse(RemoteWallClockTime isnt undefined, RemoteWallClockTime < 60*$(minWallTime), false)");
+			statements.emplace_back("  SET OnExitHoldSubCode 42");
+			statements.emplace_back("  SET OnExitHoldReason strcat(\"The job's wall clock time\", int(RemoteWallClockTime/60), \"min, is is less than the minimum specified by the job ($(minWallTime))\")");
+			statements.emplace_back("endif");
 		}
 	}
 
@@ -2267,8 +2284,8 @@ int ConvertClassadJobRouterRouteToXForm (
 	for (classad::References::const_iterator it = evalset_myrefs.begin(); it != evalset_myrefs.end(); ++it) {
 		if (assignments.find(*it) != assignments.end() && set_cmds.find(*it) == set_cmds.end()) {
 			if ( ! cSpecial) {
-				statements.append("");
-				statements.append("# temporarily SET attrs because eval_set_ rules refer to them");
+				statements.emplace_back("");
+				statements.emplace_back("# temporarily SET attrs because eval_set_ rules refer to them");
 			}
 			++cSpecial;
 			if (string_assignments.find(*it) != string_assignments.end()) {
@@ -2276,26 +2293,26 @@ int ConvertClassadJobRouterRouteToXForm (
 			} else {
 				formatstr(buf, "SET %s $(%s)", it->c_str(), it->c_str());
 			}
-			statements.append(buf.c_str());
+			statements.emplace_back(buf.c_str());
 		}
 	}
 
 	if ( ! evalset_cmds.empty()) {
-		statements.append("");
-		statements.append("# eval_set_* rules");
+		statements.emplace_back("");
+		statements.emplace_back("# eval_set_* rules");
 		for (STRING_MAP::iterator it = evalset_cmds.begin(); it != evalset_cmds.end(); ++it) {
 			formatstr(buf, "EVALSET %s %s", it->first.c_str(), it->second.c_str());
-			statements.append(buf.c_str());
+			statements.emplace_back(buf.c_str());
 		}
 	}
 
 	if (cSpecial) {
-		statements.append("");
-		statements.append("# remove temporary attrs");
+		statements.emplace_back("");
+		statements.emplace_back("# remove temporary attrs");
 		for (classad::References::const_iterator it = evalset_myrefs.begin(); it != evalset_myrefs.end(); ++it) {
 			if (assignments.find(*it) != assignments.end() && set_cmds.find(*it) == set_cmds.end()) {
 				formatstr(buf, "DELETE %s", it->c_str());
-				statements.append(buf.c_str());
+				statements.emplace_back(buf.c_str());
 			}
 		}
 	}
@@ -2310,15 +2327,15 @@ int XFormLoadFromClassadJobRouterRoute (
 	const classad::ClassAd & base_route_ad,
 	int options)
 {
-	StringList statements;
+	std::vector<std::string> statements;
 	std::string name(xform.getName());
 	int rval = ConvertClassadJobRouterRouteToXForm(statements, name, routing_string, offset, base_route_ad, options);
 	if (rval == 1) {
 		std::string errmsg;
-		auto_free_ptr xform_text(statements.print_to_delimed_string("\n"));
+		std::string xform_text = join(statements,"\n");
 		int offset = 0;
 		xform.setName(name.c_str());
-		rval = xform.open(xform_text, offset, errmsg);
+		rval = xform.open(xform_text.c_str(), offset, errmsg);
 	}
 	return rval;
 }
