@@ -20,7 +20,6 @@
 #include "condor_common.h"
 #include "condor_debug.h"
 #include "condor_config.h"
-#include "string_list.h"
 #include "condor_attributes.h"
 #include "condor_classad.h"
 #include "condor_daemon_core.h"
@@ -440,7 +439,7 @@ bool canSwitchUid(void)
 /**
  * merge_stderr_with_stdout is intended for clients of this function
  * that wish to have the old behavior, where stderr and stdout were
- * both added to the same StringList.
+ * both added to the same vector.
  */
 int systemCommand( ArgList &args, priv_state priv, std::vector<std::string> *cmd_out, std::vector<std::string> * cmd_in,
 		std::vector<std::string> *cmd_err, bool merge_stderr_with_stdout)
