@@ -21,8 +21,9 @@
 #define AMAZON_COMMANDS_H
 
 #include "condor_common.h"
-#include "string_list.h"
 
+#include <string>
+#include <vector>
 #include <map>
 
 // EC2 Commands
