@@ -46,16 +46,36 @@ struct SubmitBlob {
         // the value otherwise.
         int queueStatementCount() const;
 
-        // Given a cluster ID, parses the queue statement and initializes
-        // the itemdata variables.  Returns the corresponding SubmitForeachArgs
-        // pointer or NULL on a failure.
-        SubmitForeachArgs * init_vars( int clusterID );
-        void set_vars( StringList & vars, char * item, int itemIndex );
-        void cleanup_vars( StringList & vars );
+        SubmitForeachArgs * init_sfa();
+        void set_sfa( SubmitForeachArgs * sfa );
+        void set_vars( const std::vector<std::string> & vars, char * item, int itemIndex );
+        void cleanup_vars( const std::vector<std::string> & vars );
 
         const std::string & get_queue_args() const;
         bool set_queue_args( const char * queue_args );
 
+        void make_digest( std::string & buffer, int clusterID, const std::vector<std::string> & vars, int options ) {
+            (void) m_hash.make_digest(buffer, clusterID, vars, options);
+        }
+        bool submit_param_long_exists( const char * name, const char * alt_name, long long & value, bool int_range=false ) const {
+            return m_hash.submit_param_long_exists( name, alt_name, value, int_range );
+        }
+
+        void setSubmitMethod(int method_value) { m_hash.setSubmitMethod(method_value); }
+        int  getSubmitMethod() { return m_hash.getSubmitMethod(); }
+
+        // Something in init_vars() -- probably
+        // m_hash.load_inline_q_foreach_items() -- assumes that our
+        // "macro source" for itemdata can't be rewound.  Since we know it can,
+        // we can do so and thereby make it possible to call _submit_itemdata()
+        // more than once and without breaking a subsequent submit() call.
+        void reset_itemdata_state() { m_ms_inline.rewind_to( 0, 0 ); }
+        void insert_macro( const char * name, const std::string & value );
+
+        int process_job_credentials( std::string & URL, std::string & error_string ) {
+            return ::process_job_credentials( m_hash, 0, URL, error_string );
+        }
+
     private:
         SubmitHash m_hash;
         MACRO_SOURCE m_src_pystring;
@@ -74,6 +94,14 @@ struct SubmitBlob {
 MACRO_SOURCE SubmitBlob::EmptyMacroSrc = { false, false, 3, -2, -1, -2 };
 
 
+void
+SubmitBlob::insert_macro( const char * name, const std::string & value ) {
+    MACRO_EVAL_CONTEXT ctx;
+    ctx.init("SUBMIT");
+    ::insert_macro( name, value.c_str(), m_hash.macros(), DetectedMacro, ctx );
+}
+
+
 const char *
 SubmitBlob::lookup(const char * key) {
     // return m_hash.lookup( key );
@@ -156,7 +184,7 @@ SubmitBlob::queueStatementCount() const {
 
 
 SubmitForeachArgs *
-SubmitBlob::init_vars( int /* clusterID */ ) {
+SubmitBlob::init_sfa() {
     char * expanded_queue_args = m_hash.expand_macro( m_qargs.c_str() );
 
     SubmitForeachArgs * sfa = new SubmitForeachArgs();
@@ -179,58 +207,64 @@ SubmitBlob::init_vars( int /* clusterID */ ) {
         return NULL;
     }
 
-
-    char * var = NULL;
-    sfa->vars.rewind();
-    while( (var = sfa->vars.next()) ) {
-        // Note that this implies that updates to variables created by the
-        // queue statement MUST be pointer replacements.
-        m_hash.set_live_submit_variable( var, EmptyItemString, false );
-    }
-
-
-    m_hash.optimize();
     return sfa;
 }
 
 
 void
-SubmitBlob::set_vars( StringList & vars, char * item, int /* itemIndex */ ) {
-    if( vars.isEmpty() ) { return; }
+SubmitBlob::set_sfa( SubmitForeachArgs * sfa ) {
+    for (const auto& var: sfa->vars) {
+        // Note that this implies that updates to variables created by the
+        // queue statement MUST be pointer replacements.
+        m_hash.set_live_submit_variable( var.c_str(), EmptyItemString, false );
+    }
+
+    m_hash.optimize();
+}
+
+
+void
+SubmitBlob::set_vars( const std::vector<std::string> & vars, char * item, int /* itemIndex */ ) {
+    if( vars.empty() ) { return; }
 
     if( item == NULL ) {
         item = EmptyItemString;
     }
 
     // This is awful, but it's what condor_submit does.
-    vars.rewind();
-    char * var = vars.next();
+    auto var_it = vars.begin();
     char * data = item;
-    m_hash.set_live_submit_variable( var, data, false );
+    m_hash.set_live_submit_variable( var_it->c_str(), data, false );
 
     // This is for the human-readable form in the submit file.
     const char * separators = ", \t";
     const char * whitespace = " \t";
 
-    while( (var = vars.next()) ) {
+    // This is for when the bindings construct a submit file from itemdata.
+    if( strchr(data, '\x1F') != NULL ) {
+        separators = "\x1F";
+        // The line parser eats leading and trailing whitespace, so if we
+        // include it here, only part of it actually makes it through.
+        // Fixing this would be a pain in the ass, so let's just document
+        // if for now.
+        // whitespace = NULL;
+    }
+
+    while( ++var_it != vars.end() ) {
         while (*data && ! strchr(separators, *data)) ++data;
         if( data != NULL ) {
             *data++ = 0;
             while (*data && strchr(whitespace, *data)) ++data;
-            m_hash.set_live_submit_variable(var, data, false);
+            m_hash.set_live_submit_variable(var_it->c_str(), data, false);
         }
     }
 }
 
 
 void
-SubmitBlob::cleanup_vars( StringList & vars ) {
-    if( vars.isEmpty() ) { return; }
-
-    vars.rewind();
-    char * var = NULL;
-    while( (var = vars.next()) ) {
-        m_hash.set_live_submit_variable( var, NULL, false );
+SubmitBlob::cleanup_vars( const std::vector<std::string> & vars ) {
+    for (const auto& var: vars) {
+        m_hash.set_live_submit_variable( var.c_str(), NULL, false );
     }
 }
 
@@ -242,6 +276,7 @@ SubmitBlob::from_lines( const char * lines, std::string & errorMessage ) {
     char * qLine = NULL;
     int rv = m_hash.parse_up_to_q_line(msmf, errorMessage, &qLine);
     if( rv != 0 ) {
+        formatstr(errorMessage, "parse_up_to_q_line() failed");
         return false;
     }
 
@@ -524,6 +559,7 @@ set_dag_options( PyObject * options, DagmanOptions& dag_opts) {
     return true;
 }
 
+
 namespace shallow = DagmanShallowOptions;
 
 static PyObject *
@@ -569,6 +605,7 @@ _submit_from_dag( PyObject *, PyObject * args ) {
     return PyUnicode_FromString( dag_opts[shallow::str::SubFile].c_str() );
 }
 
+
 static PyObject *
 _display_dag_options( PyObject *, PyObject * /*args*/ ) {
     DagmanUtils du;
@@ -577,3 +614,102 @@ _display_dag_options( PyObject *, PyObject * /*args*/ ) {
     Py_RETURN_NONE;
 }
 
+
+static PyObject *
+_submit_set_submit_method( PyObject *, PyObject * args ) {
+    // _submit_set_submit_method( self._handle, method_value )
+    PyObject_Handle * handle = NULL;
+    long method_value = -1;
+
+    if(! PyArg_ParseTuple( args, "Ol", (PyObject **)& handle, & method_value )) {
+        // PyArg_ParseTuple() has already set an exception for us.
+        return NULL;
+    }
+
+    SubmitBlob * sb = (SubmitBlob *)handle->t;
+    sb->setSubmitMethod( method_value );
+
+    Py_RETURN_NONE;
+}
+
+
+static PyObject *
+_submit_get_submit_method( PyObject *, PyObject * args ) {
+    // _submit_get_submit_method( self._handle )
+    PyObject_Handle * handle = NULL;
+
+    if(! PyArg_ParseTuple( args, "O", (PyObject **)& handle )) {
+        // PyArg_ParseTuple() has already set an exception for us.
+        return NULL;
+    }
+
+    SubmitBlob * sb = (SubmitBlob *)handle->t;
+    long method_value = sb->getSubmitMethod();
+
+    return PyLong_FromLong(method_value);
+}
+
+
+static PyObject *
+_submit_itemdata( PyObject *, PyObject * args ) {
+    PyObject * self = NULL;
+    PyObject_Handle * handle = NULL;
+
+    if(! PyArg_ParseTuple( args, "OO", & self, (PyObject **)& handle )) {
+        // PyArg_ParseTuple() has already set an exception for us.
+        return NULL;
+    }
+
+    SubmitBlob * sb = (SubmitBlob *)handle->t;
+
+    SubmitForeachArgs * itemdata = sb->init_sfa();
+    sb->set_sfa(itemdata);
+
+    if( itemdata == NULL ) {
+        sb->reset_itemdata_state();
+
+        PyErr_SetString( PyExc_ValueError, "invalid Queue statement" );
+        return NULL;
+    }
+
+    if( itemdata->items.size() == 0 ) {
+        sb->reset_itemdata_state();
+
+        Py_RETURN_NONE;
+    }
+
+    std::string value = join(itemdata->items, "\n");
+
+    sb->reset_itemdata_state();
+    return PyUnicode_FromString(value.c_str());
+}
+
+
+static PyObject *
+_submit_issue_credentials( PyObject *, PyObject * args ) {
+    // _submit_issue_credentials(self.handle_t)
+
+    PyObject_Handle * handle = NULL;
+
+    if(! PyArg_ParseTuple( args, "O", (PyObject **)& handle )) {
+        // PyArg_ParseTuple() has already set an exception for us.
+        return NULL;
+    }
+
+    SubmitBlob * sb = (SubmitBlob *)handle->t;
+
+    std::string URL;
+    std::string error_string;
+    int rv = sb->process_job_credentials( URL, error_string );
+
+    if(rv != 0) {
+        PyErr_SetString( PyExc_RuntimeError, error_string.c_str() );
+        return NULL;
+    }
+
+    if(! URL.empty()) {
+        return PyUnicode_FromString(URL.c_str());
+    }
+
+    Py_RETURN_NONE;
+}
